/* ============================================================================================================================================= /
 * PROJECT:         Headphone Amp
 * FILE:            Main.c
 * LAST REV:        1.2
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F26K22
 * CREATED:         26/12/2017
 * UPDATED:         19/05/2018
 *
 * DESCRIP:         Headphone amp firmware 
 *
 * ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 * 1.2              19/05/2018          Implementato invio dati di debug via UART 2 (pin PGCLK su connettore ICSP) se abilitato in main.h
 *                                      Risolto bug accensione non voluta.
 *                                      Risolto bug esposizione scritta mute
 *                                      Risolto bug ritorno in menu system
 * 1.1              23/04/2018          Risolto bug intercettazione segnali IR random che risultavano in ripetizione comando.
 * 1.0              21/04/2018          Prima build completa
 /* ============================================================================================================================================ *
 *
/* INCLUDES =================================================================================================================================== */
#include <xc.h>
#include "Main.h"
#include "Prototipi.h"
#include "Delay.c"
#include "Manage_input.c"
#include "EEPROM.c"
#include "eeprom_map.h"
#include "Graphics.h"
#include "SPI_init.c"
#include "PGA23xx.c"
#include "OLED_SSD1306.c"

#ifdef UART_DEBUG
#include "USART_init.c"
#endif

/* GLOBAL VARIABLES =========================================================================================================================== */
// Variabili Audio
signed int master_volume = 0;                                                           
signed char balance = 0;
unsigned char input_trim = 0;
unsigned char volume_l = 0;                                                             // volume da inviare a PGA comprensivo di Balance ma senza input trim
unsigned char volume_r = 0;
bit mute = OFF;
bit zcen = OFF;
bit hp_status = 0;

// Variabili interfaccia utente
unsigned char mode = P_DOWN;
volatile signed char encoder = 0;
unsigned char input_selected = 0;
volatile bit flag_to_power_on = 0;
volatile bit flag_to_power_off = 0;
volatile bit flag_over_voltage = 0;
unsigned char contrast = 0;
unsigned char name_selected[IO_USED] = 0;
bit return_menu = 0;

// Variabili IR
volatile unsigned char ir_received[88];
//volatile bit flag_ir_received = 0;
unsigned char ir_temp[11] = {0};
volatile unsigned char ir_status = IR_IDLE;
volatile unsigned char anti_bounce = 0;
unsigned char ir_power[11];
unsigned char ir_volume_p[11];
unsigned char ir_volume_m[11];
unsigned char ir_input[11];
unsigned char ir_menu[11];
unsigned char ir_exit[11];
unsigned char ir_sharp_repeate[11] = {0xC0,0,0,0,0,0,0,0,0,0,0};                        // comando repeate per telecomandi che inviano comando di ripetizione sempre uguale
unsigned char ir_command = 0;               // comando decodificato, mantieni come variabile globale perch usato in pi punti e resettato in funzione Clear_ir()
volatile unsigned char remote_type = 1;     // 1: standard (JVC/NEC/SHARP e altri massimo 44 bit a comando) tenendo premuto viene inviato un segnale corto (8 bit)
                                            // 2: Standard (SIRC) con massimo 44 bit ricevuti tenendo premuto viene reinviato il comando
                                            // 3: Standard (SIRC) con massimo 88 bit ricevuti dove i primi 44 sono sempre uguali e vengono scartati tenendo premuto viene reinviato il comando
                                            // 4: RC5 e tutti i comandi dove cambia il 3 bit ad ogni pressione dello stesso tasto, tenendo premuto viene reinviato lo stesso comando

// Costanti scritte
#define MAX_NAMES 7
const unsigned char * input_names[MAX_NAMES] = {"Phono", "Cd", "Aux","Tv","Vcr","Pc","MD"};
const unsigned char * labels[5] = {"INPUT:", "dB", "Over", "Mute", "NO HP"};
const unsigned char * menu_items[4] = {"Mute", "Audio", "System", "Power OFF"};
const unsigned char * menu_audio_items[4] = {"Zero cross", "Balance", "Input trim","Input name"};
const unsigned char * menu_system_items[4] = {"Contrast","Remote type","Remote set","Info"};
const unsigned char * menu_values[3] = {"OFF", "ON", "->"};
const unsigned char * remote_setup_procedure[8] = {"Press button:", "POWER", "VOLUME +", "VOLUME -", "INPUT SEL", "MENU", "EXIT", "Done!"};
const unsigned char * menu_trim_items[2] = {"Input", "Level"};
const unsigned char * menu_name_items[2] = {"Input", "Name"};
const unsigned char * menu_info_items[2] = {"PCB vers.", "Fw vers."};

#define PCB_VERS    "A"
#define FW_VERS     "1.2"

/* ============================================================================================================================================= /
 * FUNCTION:        Interrupts
 *
 * Overview:        
/* ============================================================================================================================================ */
interrupt high_priority void inter(void) {
    
    // Interrupt ricezione IR
    if (INTCONbits.INT0IE && INTCONbits.INT0IF){
        INTCONbits.INT0IF = 0;
        INTCONbits.INT0IE = OFF;                                                        // disabilito interrupt  ricezione IR
        ir_status = IR_INCOMING;                                                        // flag ricezione IR, operazioni gestite in ciclo MAIN        
    }
        
    // Interrupt input over voltage
    if (INTCONbits.RBIE && INTCONbits.RBIF){                                            // abilitato interrupt solo su RB5
        if (OV_CHK == 1){                                                               // sul fronte di salita
            #ifdef REL_HEADPHONE_INSTALLED
            REL_HEADPHONE = OFF;                                                        // disabilito uscita cuffie
            #endif
            
            flag_over_voltage = 1;                                                      // flag over voltage
        }
        INTCONbits.RBIF = 0;
    }
}

interrupt low_priority void inter_low(void){
    static unsigned char _enc_a_prev = 0;
    static unsigned char _time = 0;
    
// Interrupt timer 0 per timeout anti pressione multipla da remote IR
    if (INTCONbits.TMR0IE && INTCONbits.TMR0IF){
        TMR0 = TMR0_PRELOAD;
        INTCONbits.TMR0IF = 0;
        
        anti_bounce--;
        if (anti_bounce == 0)
           T0CONbits.TMR0ON = OFF;   
    }    
    
// Interrupt timer 2 per check stato encoder con frequenza 220Hz
    if (PIE1bits.TMR2IE && PIR1bits.TMR2IF) {  
        TMR2 = TMR2_PRELOAD;
        
        if (_time < 250)
            _time++;
        
        // se l'encoder A ha cambiato stato e l'encoder B  diverso da encoder A giro orario, incremento
        // se l'encoder A ha cambiato stato e l'encoder B  uguale a encoder A (perch encoder B  cambiato prima) giro antiorario, decremento
        if (ENC_A != _enc_a_prev){
                if (ENC_B != ENC_A)  
                    encoder = 1;
                else
                    encoder = -1;
                
                if (_time >= VELO_3 && _time < VELO_2)
                    encoder *= 5;
                else if (_time >= VELO_2 && _time < VELO_1)
                    encoder *= 3;
                else if (_time >= VELO_1 && _time < VELO_0)
                    encoder *= 2;
                else if (_time >= VELO_0)
                    encoder *= 1;                
                
                _time = 0;                                                              // resetto time ad ogni impulso per contare quanto passa fino all'impulso successivo
        }

        _enc_a_prev = ENC_A;
        PIR1bits.TMR2IF = 0;
    }
    
    // interrupt pressione pulsante encoder per risveglio da sleep
    if (INTCON3bits.INT2IE && INTCON3bits.INT2IF){
        INTCON3bits.INT2IF = 0;                                                         // resetto flag
        INTCON3bits.INT2IE = 0;                                                         // e spengo interrupt                
        
        flag_to_power_on = 1;
    }
    // interrupt pressione pulsante input per risveglio da sleep
    if (INTCON3bits.INT1E && INTCON3bits.INT1IF){
        INTCON3bits.INT1IF = 0;                                                         // resetto flag
        INTCON3bits.INT1IE = 0;                                                         // e spengo interrupt                
        
        flag_to_power_on = 1;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:        Main
 *
 * Overview:        
/* ============================================================================================================================================ */

void main(void) {
    Define_IO();
    Module_settings();    
    
    Delay_s(2);  // delay per evitare accensione rel in fase di ri programmazione
    
    // reset di tutti gli input 
    D_5V_EN = 1;
    REL_IN_3 = 1;
    Delay_ms(20);
    REL_IN_3 = 0;
    D_5V_EN = 0;    
    
    // impostazione variabili default al primo avvio dopo la programmazione con reset eeprom
    if (EEPROM_read(EE_FIRST_START_ADDRESS) != 0x20)
        Save_eeprom_default();

    // caricamento variabili da eeprom all'avvio
    Load_eeprom_variables();
    Load_eeprom_remote();
    
    // inizializzazioni
    GLCD_init();
    PGA23xx_enable();
    Delay_ms(100);
    
    // ripristino impostazioni di default con i 2 tasti premuti all'avvio  
    if (ENC_SW_DIR == 0 && SEL_IN_SW_DIR == 0){
        Save_eeprom_default();  
        GLCD_set_font(font_9x15);
        GLCD_write_string(1,1,128,"Reset done!", SAVESPACE_ON, PIXEL_ON);
        GLCD_write_string(1,18,128,"Entering standby.", SAVESPACE_ON, PIXEL_ON);
        Delay_s(1);
        GLCD_write_string(1,18,128,"Entering standby..", SAVESPACE_ON, PIXEL_ON);
        Delay_s(1);
        GLCD_write_string(1,18,128,"Entering standby...", SAVESPACE_ON, PIXEL_ON);
        Delay_s(1);
        Reset();
    }
    
    // se debug abilitato manda via UART lo stato di avvio
    #ifdef UART_DEBUG
        USART2_enable(USART_BAUDR_115200);                                              // Inizializzo UART se debug abilitato
        Send_log_uart(BOOT);
    #endif
    
    // Spegnimento moduli ed entrata in sleep,
    // uscir da sleep con pressione tasti o arrivo comandi IR
    Power(OFF);
    
    // Ciclo continuo dopo primo avvio        
    while (1){
        if (ir_status == IR_INCOMING){                                                  // se  in arrivo un comando IR
            Receive_ir();                                                               // ricevo e decodifico
            Decode_ir();
        }

        // operazioni funzionamento normale
        if (mode == MAIN){
            
            if(ir_status == IR_DECODED){
                switch (ir_command){                                                    // decodifica comando remoto se ricevuto e decodificato
                    case 0:
                        break;
                    case 1: 
                        flag_to_power_off = 1;
                        break;
                    case 2:
                        encoder = 1;
                        break;
                    case 3:
                        encoder = -1;
                        break;
                    case 20:
                        encoder = 2;
                        break;
                    case 30: 
                        encoder = -2;
                        break;
                }
            }                           
                 
            /* SELEZIONE INPUT */
            if ((Debounce_single(SEL_IN_SW) || ir_command == 4) && hp_status == CONNECTED && mute == OFF){
                input_selected++;
            
                if (input_selected == IO_USED)
                    input_selected = 0;
                
                Write_input();                                                          // scrittura nome input
                Open_input(input_selected);                                             // apertura ingresso e impostazione volume
                EEPROM_write(input_selected, EE_INPUT_SEL_ADDRESS);
            }
            
            /* PRESSIONE ENCODER, ENTER MENU */
            if (Debounce_single(ENC_SW) || ir_command == 5){
                Setup();
            }
            
            // Modifica volume se  stato girato l'encoder o se sono arrivati comandi IR relativi al volume solo a cuffie connesse
            if (encoder != 0 && hp_status == CONNECTED && mute == OFF){
                
                master_volume += encoder;
                // mantengo nei limiti impostati
                if (master_volume > LIMIT_MASTER_VOL)
                    master_volume = LIMIT_MASTER_VOL;
                if (master_volume < 0)
                    master_volume = 0;
                    
                encoder = 0;
                Apply_balance(balance);
                
                if (master_volume > 0)
                    PGA23xx_send_register_value(volume_r + input_trim,volume_l + input_trim);
                else 
                    PGA23xx_send_register_value(0x00, 0x00);                            // se volume al minimo forzo scrittura volume a 0 per valori di trim > 0
                    
                Write_volume();
                EEPROM_write(master_volume, EE_VOLUME_ADDRESS);
            } else {
                encoder = 0;
            }
            
            /*
             * Uscita da menu con mute attivo
             * scrivo -99,9 e mute
             */
            if (return_menu && mute && flag_to_power_off == 0){
                return_menu = 0;
                GLCD_set_font(font_9x15);
                GLCD_write_string(LABEL_MUTE_X,LABEL_MUTE_Y,58, labels[3], SAVESPACE_ON, PIXEL_ON);    // Mute
                GLCD_set_font(font_dense_18x46);
                GLCD_write_float(VALUE_VOLUME_NEG_X,VALUE_VOLUME_Y,78,-99.9,2,1,ZERO_CLEANING_ON,SAVESPACE_ON,PIXEL_ON);
            }
            
            /*
             * Uscita da menu con cuffie disconnesse
             * scrivo -99,9 e no hp
             */
            if (return_menu && hp_status == DISCONNECTED && flag_to_power_off == 0){
                return_menu = 0;
                GLCD_set_font(font_9x15);
                GLCD_write_string(LABEL_NOHP_X,LABEL_NOHP_Y,58, labels[4], SAVESPACE_ON, PIXEL_ON);
                GLCD_set_font(font_dense_18x46);
                GLCD_write_float(VALUE_VOLUME_NEG_X,VALUE_VOLUME_Y,78,-99.9,2,1,ZERO_CLEANING_ON,SAVESPACE_ON,PIXEL_ON);
            }
            
            /*
             * Operazioni over voltage
             * scritta over su display
             * timer per riattivare ingressi e uscita cuffie
            */
            if (flag_over_voltage){

                REL_IN_3 = 1;
                Delay_ms(20);
                REL_IN_3 = 0;
                
                PGA23xx_send_register_value(0x00, 0x00);
                
                // scrittura OVER su display
                GLCD_set_font(font_9x15);
                GLCD_write_string(LABEL_OVER_X,LABEL_OVER_Y,58, labels[2], SAVESPACE_ON, PIXEL_ON);       // OVER
                
                // Controlla OV_CHK ogni 3 secondi finch il segnale resta in over voltage (DC in ingresso)
                // resta bloccato qui finch non viene eliminato il problema.
                do {
                    Delay_s(3);  
                } while (OV_CHK == 1);                                       
                
                flag_over_voltage = 0;
                GLCD_fill_rect(LABEL_OVER_X, LABEL_OVER_Y, 58, 16, PIXEL_OFF);
                                    
                // riapro ingresso e uscita    
                if (hp_status == CONNECTED){
                    Write_input();
                    Open_input(input_selected); 
                    #ifdef REL_HEADPHONE_INSTALLED
                    REL_HEADPHONE = ON;                                                     // abilito uscita cuffie
                    #endif
                } else {
                    GLCD_set_font(font_9x15);
                    GLCD_write_string(LABEL_NOHP_X,LABEL_NOHP_Y,58,  labels[4], SAVESPACE_ON, PIXEL_ON);
                }
            }
            
            /*
             * Controllo cuffie
            */
            // disconnessione cuffie
            if (HP_CHK == 0 && hp_status == CONNECTED){
                #ifdef REL_HEADPHONE_INSTALLED
                REL_HEADPHONE = OFF;                                                    // disabilito uscita cuffie
                #endif
                hp_status = DISCONNECTED;
                
                PGA23xx_send_register_value(0x00, 0x00);                                // mute
                // reset di tutti gli input 
                REL_IN_3 = 1;
                Delay_ms(20);
                REL_IN_3 = 0;
                
                // scrittura NO HP e Volume -99.9
                GLCD_set_font(font_9x15);
                GLCD_write_string(LABEL_NOHP_X,LABEL_NOHP_Y,58,  labels[4], SAVESPACE_ON, PIXEL_ON);
                GLCD_set_font(font_dense_18x46);
                GLCD_write_float(VALUE_VOLUME_NEG_X,VALUE_VOLUME_Y,78,-99.9,2,1,ZERO_CLEANING_ON,SAVESPACE_ON,PIXEL_ON);
            }
            
            // connessione cuffie
            if (HP_CHK == 1 && hp_status == DISCONNECTED){
                hp_status = CONNECTED;
                Delay_s(1);
                Write_input();
                Write_volume();
                Open_input(input_selected);                
                #ifdef REL_HEADPHONE_INSTALLED
                REL_HEADPHONE = ON;                                                     // abilito uscita cuffie
                #endif               
            }
            
            if (ir_status == IR_DECODED)
                Clear_ir_buffers();                                                     // Resetta buffer, comando ricevuto e riabilita ricezione
                
            // spegnimento se flag  impostato (da comando IR o da Menu)
            if (flag_to_power_off){
//                flag_to_power_off = 0;
                Power(OFF);
            }
        }
        
        // operazioni in stand by solo quando risvegliato
        else if (mode == P_DOWN) {
        
            #ifdef UART_DEBUG
                Send_log_uart(WAKE);
            #endif
        
            if (ir_status == IR_DECODED && ir_command == 1)                             // se ricevuto power da ir
                flag_to_power_on = 1;
            
            if (flag_to_power_on){                                                      // se flag impostato da ir o da pressione tasti
//                flag_to_power_on = 0;                                                   // avvio
                Power(ON);
            } else {                                                                    // altrimenti torno in sleep
                Clear_ir_buffers();                                                     // Resetta buffer, comando ricevuto e riabilita ricezione
                Sleep();
                Nop();                
            }
        }
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Write_main_layout
 *
 * Overview:    Scrittura su display scritte fisse per pagina MAIN
/* ============================================================================================================================================ */
void Write_main_layout(void){
    GLCD_set_font(font_9x15);
    GLCD_write_string(LABEL_INPUT_X,LABEL_INPUT_Y,45, labels[0], SAVESPACE_ON, PIXEL_ON);       // INPUT:
    GLCD_write_string(LABEL_DB_X,LABEL_DB_Y,20, labels[1], SAVESPACE_ON, PIXEL_ON);             // dB
}

/* ============================================================================================================================================= /
 * FUNCTION:    Write_volume
 *
 * Overview:    Scrittura del volume
/* ============================================================================================================================================ */
void Write_volume(void){
    unsigned char _x_dynamic_pos = VALUE_VOLUME_NEG_X;
    unsigned char _x_dynamic_clear = 78;
    float _value = 0;
    
    #ifdef REAL_GAIN
        _value = PGA23xx_calc_db(master_volume) + 6.0;                                  // aggiungo 6 per adattare a guadagno 2 dell'amplificatore finale
    #elif defined PGA2320_GAIN
        _value = PGA23xx_calc_db(master_volume);                                        
    #endif
    
    if (_value >= 0) {                                                                  // valori positivi la scritta va spostata a destra
        _x_dynamic_pos = VALUE_VOLUME_POS_X;
        _x_dynamic_clear -= 14;
        GLCD_fill_rect(VALUE_VOLUME_NEG_X,VALUE_VOLUME_Y + 25,14,3,PIXEL_OFF);          // cancello segno meno
    }
    
    GLCD_set_font(font_dense_18x46);
    GLCD_write_float(_x_dynamic_pos,VALUE_VOLUME_Y,_x_dynamic_clear,_value,2,1,ZERO_CLEANING_ON,SAVESPACE_ON,PIXEL_ON);
    
}

/* ============================================================================================================================================= /
 * FUNCTION:    Write_input
 *
 * Overview:    Scrittura nome input
/* ============================================================================================================================================ */
void Write_input(void){
    GLCD_set_font(font_9x15);
    GLCD_write_string(VALUE_INPUT_X,VALUE_INPUT_Y,60, input_names[name_selected[input_selected]] , SAVESPACE_ON, PIXEL_ON);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup
 *
 * Overview:    Gestione delle impostazioni.
 *              Resta in questo loop fino a che non viene premuto il tasto input (exit) o non viene ricevuto il relativo comando remoto.
 * Todo:        Timer per uscita automatica
/* ============================================================================================================================================ */
void Setup(void){
    unsigned char _sel = 0;
    unsigned char _input_selected_backup = 0;
    
    _input_selected_backup = input_selected; 
    
    // scrivo i valori del menu e attendo una selezione
    _sel = Handle_menu(4,0,MAIN_MENU,menu_items);
    
    do{        
        switch (_sel){
            /* MENU PRINCIPALE */
            case 0:                                                                     // mute on off 
                mute = !mute;
                switch (mute){
                    case ON:
                        PGA23xx_send_register_value(0x00,0x00);
                        break;
                    case OFF:
                        PGA23xx_send_register_value(volume_r + input_trim,volume_l + input_trim);
                        break;
                }
            
                _sel = Handle_menu(4,0,MAIN_MENU,menu_items);                           // riscrivo menu principale e attendo nuova selezione
                break;
            case 1:                                                                     // menu audio
                _sel = Handle_menu(4,0,AUDIO_MENU,menu_audio_items);                    // disegno il menu audio e attendo selezione
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,1,MAIN_MENU,menu_items);                       // torno al menu principale con selezione su audio
                break;
            case 2:                                                                     // menu system
                _sel = Handle_menu(4,0,SYSTEM_MENU,menu_system_items);                  // disegno il menu system e attendo selezione
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,2,MAIN_MENU,menu_items);                       // torno al menu principale con selezione su system
                break;
            case 3:                                                                     // power off
                Delay_ms(200);
                flag_to_power_off = 1;
                _sel = 0xFE;                                                            // forzo uscita senza riscrittura della pagina main        
                break;
            
            /* MENU AUDIO */
            case 4:                                                                     // zero cross on off   
                zcen = !zcen;
                PGA23xx_zcen(zcen);
                EEPROM_write(zcen,EE_ZCEN_ADDRESS);
                _sel = Handle_menu(4,0,AUDIO_MENU,menu_audio_items);                    // Ridisegno il menu audio con selezione su zero crossing
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,1,MAIN_MENU,menu_items);                       // torno al menu principale con selezione su audio
                break;
            case 5:                                                                     // balance
                Setup_balance();
                _sel = Handle_menu(4,1,AUDIO_MENU,menu_audio_items);
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,1,MAIN_MENU,menu_items);                       // torno al menu principale con selezione su audio
                break;
            case 6:                                                                     // input trim
                _sel = Handle_menu(2,0,TRIM_MENU,menu_trim_items);
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,2,AUDIO_MENU,menu_audio_items);
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,1,MAIN_MENU,menu_items);
                
                break;
            case 7:
                _sel = Handle_menu(2,0,NAME_MENU,menu_name_items);
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,3,AUDIO_MENU,menu_audio_items);
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,1,MAIN_MENU,menu_items);

                break;                
            
            /* MENU SYSTEM */    
            case 8:                                                                     // contrast
                Setup_contrast();
                _sel = Handle_menu(4,0,SYSTEM_MENU,menu_system_items);
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,2,MAIN_MENU,menu_items);                       // torno al menu principale con selezione su system
                break;
            case 9:                                                                     // remote type
                remote_type++;                                                          // incremento remote type mantenendolo nei limiti
                if (remote_type > MAX_REMOTE_TYPE)
                    remote_type = 1;
                EEPROM_write(remote_type,EE_REMOTE_TYPE_ADDRESS);                       // salvo selezione in eeprom
                _sel = Handle_menu(4,1,SYSTEM_MENU,menu_system_items);                  // torno a menu system con selezione su remote type
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,2,MAIN_MENU,menu_items);                       // torno al menu principale con selezione su system
                break;
            case 10:                                                                    // remote set
                Setup_remote();                                                         // registrazione comandi remoti
                _sel = Handle_menu(4,2,SYSTEM_MENU,menu_system_items);
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,2,MAIN_MENU,menu_items);                       // torno al menu principale con selezione su system
                break;
            case 11:                                                                    // info menu
                Handle_static_info(2,INFO,menu_info_items); 
                _sel = Handle_menu(4,3,SYSTEM_MENU,menu_system_items);
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,2,MAIN_MENU,menu_items);                       // torno al menu principale con selezione su system
                break;
            
            /* SETTAGGIO INPUT TRIM SELEZIONE INGRESSO */                         
            case 12:                                                                    // selezione ingresso per input trim
                input_selected++;
            
                if (input_selected == IO_USED)
                    input_selected = 0;
                                
                Open_input(input_selected);
             
                _sel = 6;
                
                break;
                
            /* SETTAGGIO INPUT TRIM VALORE TRIM */
            case 13:                                                                    // regolazione trim per ingresso selezionato
                Setup_trim();
                _sel = Handle_menu(2,1,TRIM_MENU,menu_trim_items);
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF                                                  // se selezione 0xFF
                    _sel = Handle_menu(4,2,AUDIO_MENU,menu_audio_items);
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,1,MAIN_MENU,menu_items);
                break;
            
            /* SETTAGGIO INPUT NAME SELEZIONE INGRESSO */ 
            case 16:                                                                    // selezione ingresso per input name
                input_selected++;
            
                if (input_selected == IO_USED)
                    input_selected = 0;
                
                _sel = 7;
                break;                
            
            /* SETTAGGIO INPUT NAME NOME INGRESSO */   
            case 17:
                Setup_name();
                _sel = Handle_menu(2,1,NAME_MENU,menu_name_items);
                
                if (_sel == 0xFF)                                                       // se selezione 0xFF                                                  // se selezione 0xFF
                    _sel = Handle_menu(4,3,AUDIO_MENU,menu_audio_items);
                if (_sel == 0xFF)                                                       // se selezione 0xFF
                    _sel = Handle_menu(4,1,MAIN_MENU,menu_items);
                break;
                
        }
    }while (_sel != 0xFF && _sel != 0xFE);                                              // ciclo finch sel non  0xFF o 0xFE
        if (_sel == 0xFF){
            if (input_selected != _input_selected_backup) {                             // riporto all'input pre setup se  stato modificato      
                input_selected = _input_selected_backup;
                Open_input(input_selected);
//                PGA23xx_send_register_value(volume_r + input_trim,volume_l + input_trim);   // reimposto il volume
            }
            GLCD_clear(PIXEL_OFF);
            Write_main_layout();
            Write_input();
            Write_volume();
        }
    
    return_menu = 1;
}

/* ============================================================================================================================================= /
 * FUNCTION:    Handle_static_info
 * Input:       _items_number: numero di righe da scrivere nel menu
 *              _menu_number: numero del menu
 *              * _items[]: array contenente le scritte da scrivere
 *              
 * Overview:    Scrive su display gli item del menu selezionato e attende fino a che non viene premuto il tasto encoder o il tasto input       
 *              o non vengono ricevuti i relativi comandi remoti.
/* ============================================================================================================================================ */
void Handle_static_info(unsigned char _items_number, unsigned char _menu_number, const unsigned char * _items[]){
    unsigned char _exit = 0;
            
    Clear_ir_buffers();
    
    GLCD_clear(PIXEL_OFF);
    GLCD_set_font(font_9x15);
    
    Write_menu_values(_menu_number);
    
    // scrivo tutte le righe del menu
    for (unsigned char _i = 0; _i < _items_number; _i++){
        GLCD_write_string(LABEL_MENU_IT_X,(16*_i)+1,60, _items[_i] , SAVESPACE_ON, PIXEL_ON);
    }
    
    // ciclo fino a pressione tasto encoder o tasto selezione input 
    do {
        
        if (ir_status == IR_INCOMING){
            Receive_ir();
            Decode_ir();
        }    
        
        if (Debounce_single(ENC_SW) || ir_command == 5)
            _exit = 1;
        if (Debounce_single(SEL_IN_SW) || ir_command == 6)
            _exit = 2;
            
        if (ir_status == IR_DECODED)
            Clear_ir_buffers();                                                         // Resetta buffer, comando ricevuto e riabilita ricezione
            
    } while (_exit == 0);
} 

/* ============================================================================================================================================= /
 * FUNCTION:    Handle_menu
 * Input:       _items_number: numero di righe da scrivere nel menu
 *              _selection: selezione iniziale
 *              _menu_number: numero del menu
 *              * _items[]: array contenente le scritte da scrivere
 * Output:      numero della riga selezionata moltiplicata per il numero del menu:
 *              0~3: righe del menu 1
 *              4~7: righe del menu 2
 *              ...
 *              0xFF: tasto exit premuto
 *              
 * Overview:    Scrive su display gli item del menu selezionato e monitora le selezioni con encoder o comandi remoti.
 *              resta in questo loop fino a che non viene premuto il tasto encoder (seleziona o modifica valore) o il tasto input (exit)       
 *              o non vengono ricevuti i relativi comandi remoti.
/* ============================================================================================================================================ */
unsigned char Handle_menu(unsigned char _items_number, unsigned char _selection, unsigned char _menu_number, const unsigned char * _items[]){
    unsigned char _exit = 0;
    unsigned char _prev_selection = 0;
    unsigned char _return = 0xFF;
    unsigned char _sel_rect_l = MENU_SEL_RECT_L;
    
    Clear_ir_buffers();
    
    if (_menu_number == TRIM_MENU || _menu_number == NAME_MENU)
        _sel_rect_l = MENU_TRIM_SEL_RECT_L;
    
    GLCD_clear(PIXEL_OFF);
    GLCD_set_font(font_9x15);
    
    Write_menu_values(_menu_number);    
  
    
    // scrivo tutte le righe del menu
    for (unsigned char _i = 0; _i < _items_number; _i++){
        GLCD_write_string(LABEL_MENU_IT_X,(16*_i)+1,60, _items[_i] , SAVESPACE_ON, PIXEL_ON);
    }

    // scrivo la riga selezionata in negativo
    GLCD_draw_line_y((16*_selection)+1,(16*_selection)+16,LABEL_MENU_IT_X-1,PIXEL_ON);
    GLCD_write_string(LABEL_MENU_IT_X,(16*_selection)+1,_sel_rect_l, _items[_selection] , SAVESPACE_ON, PIXEL_OFF);
    
    // ciclo fino a pressione tasto encoder (salva) o tasto selezione input (esci)
    do {
        if (ir_status == IR_INCOMING){
            Receive_ir();
            Decode_ir();
        
            switch (ir_command){
                case 0:
                    break;
                case 2:
                    encoder = -1;
                    break;
                case 3:
                    encoder = 1;
                    break;
            }   
        }
        
        if (encoder != 0){
            
            if (encoder > 0)
                encoder = 1;
            else
                encoder = -1;
            
            _prev_selection = _selection;
            _selection += encoder;
            encoder = 0;
            
            // limite prima e ultima riga
            if (_selection == _items_number)
                _selection = _items_number -1;
            else if (_selection == 0xFF)
                _selection = 0;
            else {
                // scrivo vecchia selezione normale e nuova selezione in negativo
                GLCD_draw_line_y((16*_prev_selection)+1,(16*_prev_selection)+16,LABEL_MENU_IT_X-1,PIXEL_OFF);
                GLCD_write_string(LABEL_MENU_IT_X,(16*_prev_selection)+1,_sel_rect_l, _items[_prev_selection] , SAVESPACE_ON, PIXEL_ON);
                GLCD_draw_line_y((16*_selection)+1,(16*_selection)+16,LABEL_MENU_IT_X-1,PIXEL_ON);
                GLCD_write_string(LABEL_MENU_IT_X,(16*_selection)+1,_sel_rect_l, _items[_selection] , SAVESPACE_ON, PIXEL_OFF);
            }
        }    
        
        if (Debounce_single(ENC_SW) || ir_command == 5)
            _exit = 1;
        if (Debounce_single(SEL_IN_SW) || ir_command == 6)
            _exit = 2;
            
        if (ir_status == IR_DECODED)
            Clear_ir_buffers();                                                         // Resetta buffer, comando ricevuto e riabilita ricezione
            
    } while (_exit == 0);
    
    // uscita con tasto encoder, altrimenti return resta 0xFF (uscita con tasto selezione input)
    if (_exit == 1)
        _return = ((_menu_number * 4) +_selection);

    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    Write_menu_values
 * Input:       _menu_number: numero del menu
 *              
 * Overview:    Scrive su display il valore delle impostazioni a seconda della pagina menu visualizzata
/* ============================================================================================================================================ */
void Write_menu_values (unsigned char _menu_number){
    
    switch (_menu_number){
        case MAIN_MENU:                                                                                     // menu principale
            GLCD_write_string(VALUE_MENU_X,MENU_Y_1,38, menu_values[mute], SAVESPACE_ON, PIXEL_ON);         // Mute: On/off
            GLCD_write_string(VALUE_MENU_X,MENU_Y_2,38, menu_values[2], SAVESPACE_ON, PIXEL_ON);            // Audio: ->
            GLCD_write_string(VALUE_MENU_X,MENU_Y_3,38, menu_values[2], SAVESPACE_ON, PIXEL_ON);            // System: ->
            break;
        case AUDIO_MENU:
            GLCD_write_string(VALUE_MENU_X,MENU_Y_1,38, menu_values[zcen], SAVESPACE_ON, PIXEL_ON);         // Zero cross: On/off
            GLCD_write_string(VALUE_MENU_X,MENU_Y_2,38, menu_values[2], SAVESPACE_ON, PIXEL_ON);            // Balance: ->
            GLCD_write_string(VALUE_MENU_X,MENU_Y_3,38, menu_values[2], SAVESPACE_ON, PIXEL_ON);            // Input trim: ->
            GLCD_write_string(VALUE_MENU_X,MENU_Y_4,38, menu_values[2], SAVESPACE_ON, PIXEL_ON);            // Input name  ->
            break;
        case SYSTEM_MENU:
            GLCD_write_string(VALUE_MENU_X,MENU_Y_1,38, menu_values[2], SAVESPACE_ON, PIXEL_ON);            // Contrast: ->
            GLCD_write_unsint(VALUE_MENU_X,MENU_Y_2,38, remote_type,1,ZERO_CLEANING_OFF, SAVESPACE_ON, PIXEL_ON);    // Remote type: 1/2
            GLCD_write_string(VALUE_MENU_X,MENU_Y_3,38, menu_values[2], SAVESPACE_ON, PIXEL_ON);            // Remote set  ->
            break;
        case TRIM_MENU:
            GLCD_write_string(TRIM_VALUE_X,MENU_Y_1,40, input_names[name_selected[input_selected]], SAVESPACE_ON, PIXEL_ON);           // Input selezionato
            GLCD_write_float(TRIM_VALUE_X,MENU_Y_2,40,(0.5*input_trim),2,1,ZERO_CLEANING_ON,SAVESPACE_ON,PIXEL_ON);
            GLCD_write_string(TRIM_VALUE_X + 30,MENU_Y_2,20, labels[1], SAVESPACE_ON, PIXEL_ON);            // dB
            break; 
        case NAME_MENU:
            GLCD_write_unsint(NAME_VALUE_X,MENU_Y_1,38, input_selected + 1,1,ZERO_CLEANING_OFF, SAVESPACE_ON, PIXEL_ON);    // numero input
            GLCD_write_string(NAME_VALUE_X,MENU_Y_2,40, input_names[name_selected[input_selected]], SAVESPACE_ON, PIXEL_ON);           // Input selezionato
            break;       
        case INFO:
            GLCD_write_string(VALUE_MENU_X,MENU_Y_1,38, PCB_VERS, SAVESPACE_ON, PIXEL_ON); 
            GLCD_write_string(VALUE_MENU_X,MENU_Y_2,38, FW_VERS, SAVESPACE_ON, PIXEL_ON);
            break;
    }
} 

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_contrast
 *              
 * Overview:    Imposta il contrasto del display. tasto encoder salva il nuovo valore, tasto input exit senza salvare
/* ============================================================================================================================================ */
void Setup_contrast(void){
    unsigned char _exit = 0;
    signed int _contrast = 0;
    
    Clear_ir_buffers();
    
    _contrast = contrast;                                                               // valore temporaneo
    
    GLCD_clear(PIXEL_OFF);
    GLCD_set_font(font_9x15);
    GLCD_write_string(LABEL_MENU_IT_X,MENU_Y_1,60, menu_system_items[0] , SAVESPACE_ON, PIXEL_ON);
    GLCD_set_font(font_dense_18x46);
    
    GLCD_write_unsint(SETUP_CONTRAST_X,SETUP_CONTRAST_Y, 38, _contrast, 3, ZERO_CLEANING_ON, SAVESPACE_ON, PIXEL_ON);
    
    do{
        if (ir_status == IR_INCOMING){
            Receive_ir();
            Decode_ir();
        
            switch (ir_command){                                                        // modifico valore encoder se ricevuto ir_command
                case 0:
                    break;
                case 2:
                    encoder = 1;
                    break;
                case 3:
                    encoder = -1;
                    break;
                case 20:
                    encoder = 2;
                    break;
                case 30: 
                    encoder = -2;
                    break;
            }   
        }
        
        if (encoder != 0){                                                              // adatto il nuovo valore con rotazione encoder o comando ir
            _contrast += encoder;
            if (_contrast > 255)
                _contrast = 255;
            if (_contrast < 0)
                _contrast = 0;
            encoder = 0;

            GLCD_write_unsint(SETUP_CONTRAST_X,SETUP_CONTRAST_Y, 80, _contrast, 3, ZERO_CLEANING_ON, SAVESPACE_ON, PIXEL_ON);
            GLCD_set_contrast(_contrast);                                               // imposto il contrasto per anteprima
        }
        
        if (Debounce_single(ENC_SW) || ir_command == 5)
            _exit = 1;
        if (Debounce_single(SEL_IN_SW) || ir_command == 6)
            _exit = 2;
            
        if (ir_status == IR_DECODED)
            Clear_ir_buffers();                                                         // Resetta buffer, comando ricevuto e riabilita ricezione
                        
    } while (_exit == 0);
    
    if (_exit == 1){                                                                    // exit con pulsante encoder
        contrast = _contrast;                                                           // salvo il nuovo valore
        EEPROM_write(contrast, EE_CONTRAST_ADDRESS);
        
    } else {                                                                            // uscita con pulsante input
        GLCD_set_contrast(contrast);                                                    // ripristino il vecchio valore
    } 
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_trim
 *              
 * Overview:    Imposta il trim dell'ingresso selezionato. tasto encoder salva il nuovo valore, tasto input exit senza salvare
/* ============================================================================================================================================ */
void Setup_trim(void){
    unsigned char _exit = 0;
    signed int _input_trim = 0;
    
    Clear_ir_buffers();
    
    _input_trim = input_trim;                                                           // valore temporaneo
    
    GLCD_clear(PIXEL_OFF);
    GLCD_set_font(font_9x15);
    GLCD_write_string(LABEL_MENU_IT_X,MENU_Y_1,60, menu_trim_items[0], SAVESPACE_ON, PIXEL_ON);
    GLCD_write_string(TRIM_VALUE_X,MENU_Y_1,40, input_names[name_selected[input_selected]], SAVESPACE_ON, PIXEL_ON);           // Input selezionato
    GLCD_write_string(LABEL_MENU_IT_X,MENU_Y_2,60, menu_trim_items[1], SAVESPACE_ON, PIXEL_ON);
    GLCD_write_float(TRIM_VALUE_X,MENU_Y_2,30,(0.5*input_trim),2,1,ZERO_CLEANING_ON,SAVESPACE_ON,PIXEL_OFF);
    GLCD_write_string(TRIM_VALUE_X + 30,MENU_Y_2,20, labels[1], SAVESPACE_ON, PIXEL_OFF);             // dB
  
    
    do{
        if (ir_status == IR_INCOMING){
            Receive_ir();
            Decode_ir();
        
            switch (ir_command){                                                        // modifico valore encoder se ricevuto ir_command
                case 0:
                    break;
                case 2:
                    encoder = 1;
                    break;
                case 3:
                    encoder = -1;
                    break;
                case 20:
                    encoder = 2;
                    break;
                case 30: 
                    encoder = -2;
                    break;
            }     
        }
        
        if (encoder != 0){                                                              // adatto il nuovo valore con rotazione encoder o comando ir
            _input_trim += encoder;
            if (_input_trim > LIMIT_TRIM)
                _input_trim = LIMIT_TRIM;
            if (_input_trim < 0)
                _input_trim = 0;
            encoder = 0;

            GLCD_write_float(TRIM_VALUE_X,MENU_Y_2,30,(0.5*_input_trim),2,1,ZERO_CLEANING_ON,SAVESPACE_ON,PIXEL_OFF);
            PGA23xx_send_register_value(volume_r + _input_trim,volume_l + _input_trim);   // reimposto il volume per adattare input trim
        }
        
        if (Debounce_single(ENC_SW) || ir_command == 5)
            _exit = 1;
        if (Debounce_single(SEL_IN_SW) || ir_command == 6)
            _exit = 2;
            
        if (ir_status == IR_DECODED)
            Clear_ir_buffers();                                                         // Resetta buffer, comando ricevuto e riabilita ricezione
                        
    } while (_exit == 0);
    
    if (_exit == 1){                                                                    // exit con pulsante encoder
        input_trim = _input_trim;                                                       // salvo il nuovo valore
        EEPROM_write(input_trim, EE_INPUT_TRIM_ADDRESS + input_selected);
        
    } else {                                                                            // uscita con pulsante input
        PGA23xx_send_register_value(volume_r + input_trim,volume_l + input_trim);       // reimposto il volume per adattare input trim                                                    // ripristino il vecchio valore
    } 
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_name
 *              
 * Overview:    Imposta il nome dell'ingresso selezionato. tasto encoder salva il nuovo valore, tasto input exit senza salvare
/* ============================================================================================================================================ */
void Setup_name(void){     
    unsigned char _exit = 0;
    signed char _name_selected = 0;
    unsigned char _name_start = 0;
    
    _name_selected = name_selected[input_selected];
    _name_start = _name_selected;
            
    Clear_ir_buffers();
    
    GLCD_clear(PIXEL_OFF);
    GLCD_set_font(font_9x15);
    GLCD_write_string(LABEL_MENU_IT_X,MENU_Y_1,60, menu_name_items[0], SAVESPACE_ON, PIXEL_ON);
    GLCD_write_unsint(NAME_VALUE_X,MENU_Y_1,38, input_selected + 1,1,ZERO_CLEANING_OFF, SAVESPACE_ON, PIXEL_ON);    // numero input
    GLCD_write_string(LABEL_MENU_IT_X,MENU_Y_2,60, menu_name_items[1], SAVESPACE_ON, PIXEL_ON);                     
    GLCD_write_string(NAME_VALUE_X,MENU_Y_2,40, input_names[_name_selected], SAVESPACE_ON, PIXEL_OFF);              // Input selezionato
  
    do{
        if (ir_status == IR_INCOMING){
            Receive_ir();
            Decode_ir();
        
        
            switch (ir_command){                                                        // modifico valore encoder se ricevuto ir_command
                case 0:
                    break;
                case 2:
                    encoder = 1;
                    break;
                case 3:
                    encoder = -1;
                    break;
                case 20:
                    encoder = 2;
                    break;
                case 30: 
                    encoder = -2;
                    break;
            }          
        }
        
        
        if (encoder != 0){                                                              // adatto il nuovo valore con rotazione encoder o comando ir
            _name_selected += encoder;
            if (_name_selected >= MAX_NAMES)
                _name_selected = 0;
            if (_name_selected < 0)
                _name_selected = MAX_NAMES-1;
            encoder = 0;

            GLCD_write_string(NAME_VALUE_X,MENU_Y_2,40, input_names[_name_selected], SAVESPACE_ON, PIXEL_OFF);                    // Input selezionato
        }
        
        if (Debounce_single(ENC_SW) || ir_command == 5)
            _exit = 1;
        if (Debounce_single(SEL_IN_SW) || ir_command == 6)
            _exit = 2;
            
        if (ir_status == IR_DECODED)
            Clear_ir_buffers();                                                         // Resetta buffer, comando ricevuto e riabilita ricezione
                        
    } while (_exit == 0);
    
    if (_exit == 1){                                                                    // exit con pulsante encoder
        name_selected[input_selected] = _name_selected;
        EEPROM_write(_name_selected, EE_INPUT_NAME_ADDRESS + input_selected);   
    } else {
        GLCD_write_string(NAME_VALUE_X,MENU_Y_2,40, input_names[_name_start], SAVESPACE_ON, PIXEL_ON);                    // Input selezionato
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_remote
 *              
 * Overview:    Procedura guidata per il salvataggio dei comandi IR
/* ============================================================================================================================================ */
void Setup_remote(void){
    unsigned char skip = 0;
    
    GLCD_clear(PIXEL_OFF);
    GLCD_set_font(font_9x15);
    
    GLCD_write_string(SETUP_REMOTE_LABEL_X,SETUP_REMOTE_LABEL_Y,128,remote_setup_procedure[0], SAVESPACE_ON, PIXEL_ON);
    Delay_s(1);
    
    // ciclo per i 6 comandi da registrare
    for (unsigned char _i = 1; _i< 7; _i++){
        Clear_ir_buffers();                                                             // Resetta buffer, comando ricevuto e riabilita ricezione
        
        GLCD_write_string(SETUP_REMOTE_LABEL_X,SETUP_REMOTE_LABEL_Y,128,remote_setup_procedure[0], SAVESPACE_ON, PIXEL_ON);
        GLCD_write_string(SETUP_REMOTE_STEPS_X,SETUP_REMOTE_STEPS_Y,100,remote_setup_procedure[_i], SAVESPACE_ON, PIXEL_ON);
        
        // attendo ricezione comando ir o pressione pulsante selezione ingresso per annullare
        do{
            if (Debounce_single(SEL_IN_SW)){
                skip = 1;
                break;                                                                  // interrompo attesa segnale IR
            }
        }while (ir_status == IR_IDLE);
        
        if (skip) break;                                                                // interrompo ciclo for per uscita forzata
    
        Receive_ir();
    
        INTCONbits.INT0IE = OFF;                                                        // disabilito interrupt da segnale IR
         
        // salvataggio comando in variabili globali
        switch (_i){
            case 1: 
                Copy_array(ir_power, ir_temp);
                break;
            case 2: 
                Copy_array(ir_volume_p, ir_temp);
                break;
            case 3: 
                Copy_array(ir_volume_m, ir_temp);
                break;
            case 4: 
                Copy_array(ir_input, ir_temp);
                break;                
            case 5: 
                Copy_array(ir_menu, ir_temp);
                break;
            case 6: 
                Copy_array(ir_exit, ir_temp);
                break;
        }

        Delay_ms(250);
        // disegno immagine check attesa e cancellazione immagine
        GLCD_draw_image(Check);
        Delay_s(1);
        GLCD_clear(PIXEL_OFF);
    }
    
    Clear_ir_buffers();                                                                 // Resetta buffer, comando ricevuto e riabilita ricezione
    
    GLCD_clear(PIXEL_OFF);
    if (skip == 0){                                                                     // se la procedura  completa scrivo messaggio e salvo in eeprom
        GLCD_write_string(SETUP_REMOTE_LABEL_X,SETUP_REMOTE_LABEL_Y,100,remote_setup_procedure[7], SAVESPACE_ON, PIXEL_ON);
        Save_eeprom_remote();
        Delay_s(1);
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_balance
 *              
 * Overview:    Procedura per l'impostazione del balance
/* ============================================================================================================================================ */
void Setup_balance(void){
    unsigned char _exit = 0;
    signed int _balance = 0;
    
    Clear_ir_buffers();
    
    _balance = balance;                                                                 // valore temporaneo
    
    // disegno base
    GLCD_clear(PIXEL_OFF);
    GLCD_set_font(font_9x15);
    GLCD_write_string(LABEL_BALANCE_X,MENU_Y_1,60, menu_audio_items[1] , SAVESPACE_ON, PIXEL_ON);
    GLCD_write_char(BALANCE_L_X,BALANCE_LEGEND_Y, 'L', SAVESPACE_ON, OVERWRITE, PIXEL_ON);
    GLCD_write_char(BALANCE_R_X,BALANCE_LEGEND_Y, 'R', SAVESPACE_ON, OVERWRITE, PIXEL_ON);
    
    // barra e valore allo stato attuale    
    if (_balance == 0){
        GLCD_fill_rect(BALANCE_BAR_X+51,BALANCE_BAR_Y,2,BALANCE_BAR_H+1, PIXEL_ON);
        GLCD_write_char(BALANCE_C_X,BALANCE_VALUE_Y,'C',SAVESPACE_ON,OVERWRITE,PIXEL_ON);        
        }         
    else if (_balance > 0){
        GLCD_fill_rect(BALANCE_BAR_X+51,BALANCE_BAR_Y,2+_balance,BALANCE_BAR_H+1, PIXEL_ON);
        GLCD_write_unsint(BALANCE_R_VAL_X,BALANCE_VALUE_Y,20, _balance,2,ZERO_CLEANING_ON, SAVESPACE_ON, PIXEL_ON);        
        }        
    else {
        GLCD_fill_rect(BALANCE_BAR_X+51+_balance,BALANCE_BAR_Y,3+ (~ _balance),BALANCE_BAR_H+1, PIXEL_ON);
        GLCD_write_unsint(BALANCE_L_VAL_X,BALANCE_VALUE_Y,20, (~_balance) +1,2,ZERO_CLEANING_ON, SAVESPACE_ON, PIXEL_ON);        
        }        
    
//    Draw_balance(_balance);
    
    do{
        if (ir_status == IR_INCOMING){
            Receive_ir();
            Decode_ir();
        
            switch (ir_command){                                                        // modifico valore encoder se ricevuto ir_command
                case 0:
                    break;
                case 2:
                    encoder = 1;
                    break;
                case 3:
                    encoder = -1;
                    break;
                case 20:
                    encoder = 1;
                    break;
                case 30:
                    encoder = -1;
                    break;
            }             
        }              
        
        
        if (encoder != 0){                                                              // adatto il nuovo valore con rotazione encoder o comando ir
            if (encoder > 1)
                encoder = 1;
            if (encoder < -1)
                encoder = -1;
            
            _balance += encoder;
            if (_balance > 50)
                _balance = 50;
            if (_balance < -50)
                _balance = -50;
            encoder = 0;


            Apply_balance(_balance);                                                    // applico balance per anteprima
            PGA23xx_send_register_value(volume_r + input_trim,volume_l + input_trim);

            Draw_balance(_balance);
        }
        
        if (Debounce_single(ENC_SW) || ir_command == 5)
            _exit = 1;
        if (Debounce_single(SEL_IN_SW) || ir_command == 6)
            _exit = 2;
            
        if (ir_status == IR_DECODED)
            Clear_ir_buffers();                                                         // Resetta buffer, comando ricevuto e riabilita ricezione
                        
    } while (_exit == 0);
    
    if (_exit == 1){                                                                    // exit con pulsante encoder
        balance = _balance;                                                             // salvo il nuovo valore
        EEPROM_write(balance, EE_BALANCE_ADDRESS);
        
    } else {                                                                            // uscita con pulsante input
        Apply_balance(balance);                                                         // ripristino balance originale
        PGA23xx_send_register_value(volume_r + input_trim,volume_l + input_trim);
    } 
}

/* ============================================================================================================================================= /
 * FUNCTION:    Draw_balance
 *              
 * Overview:    Scrittura valore balance e rappresentazione grafica
/* ============================================================================================================================================ */
void Draw_balance(signed char _balance){
    static signed char _balance_prev = 0;
    
    if (_balance > 0){
        GLCD_fill_rect(BALANCE_C_X,BALANCE_VALUE_Y,10,BALANCE_VALUE_Y+16, PIXEL_OFF);   // cancella eventuale C
        if (_balance >= _balance_prev){
            GLCD_draw_line_y(BALANCE_BAR_Y,BALANCE_BAR_Y+BALANCE_BAR_H,BALANCE_BAR_X+52+_balance, PIXEL_ON);
        } else {
            GLCD_draw_line_y(BALANCE_BAR_Y,BALANCE_BAR_Y+BALANCE_BAR_H,BALANCE_BAR_X+52+_balance_prev, PIXEL_OFF);
        }
        GLCD_write_unsint(BALANCE_R_VAL_X,BALANCE_VALUE_Y,20, _balance,2,ZERO_CLEANING_ON, SAVESPACE_ON, PIXEL_ON);
    } else if (_balance < 0) {
        GLCD_fill_rect(BALANCE_C_X,BALANCE_VALUE_Y,10,BALANCE_VALUE_Y+16, PIXEL_OFF);   // cancella eventuale C
        if (_balance <= _balance_prev){
            GLCD_draw_line_y(BALANCE_BAR_Y,BALANCE_BAR_Y+BALANCE_BAR_H,BALANCE_BAR_X+51+_balance, PIXEL_ON);
        } else {
            GLCD_draw_line_y(BALANCE_BAR_Y,BALANCE_BAR_Y+BALANCE_BAR_H,BALANCE_BAR_X+51+_balance_prev, PIXEL_OFF);
        }
        GLCD_write_unsint(BALANCE_L_VAL_X,BALANCE_VALUE_Y,20, (~_balance) +1,2,ZERO_CLEANING_ON, SAVESPACE_ON, PIXEL_ON);
    } else  {
        GLCD_draw_line_y(BALANCE_BAR_Y,BALANCE_BAR_Y+BALANCE_BAR_H,BALANCE_BAR_X+50, PIXEL_OFF);
        GLCD_draw_line_y(BALANCE_BAR_Y,BALANCE_BAR_Y+BALANCE_BAR_H,BALANCE_BAR_X+53, PIXEL_OFF);
        GLCD_fill_rect(BALANCE_L_X,BALANCE_VALUE_Y,104,BALANCE_VALUE_Y+16, PIXEL_OFF);  // cancella valori di l e r
        GLCD_write_char(BALANCE_C_X,BALANCE_VALUE_Y,'C',SAVESPACE_ON,OVERWRITE,PIXEL_ON);
    }
        
    _balance_prev = _balance; 
}

/* ============================================================================================================================================= /
 * FUNCTION:    Apply_balance
 * Input:       _balance: livello da applicare
 *              
 * Overview:    Applica il livello di balance al livello generale del volume e imposta i volumi volume_r e volume_l
/* ============================================================================================================================================ */
void Apply_balance(signed char _balance){
    signed char _balance_temp = 0;
    if (_balance < 0) {
        _balance_temp = (~_balance) +1;
        volume_r = master_volume - ((master_volume*_balance_temp)/50);
        volume_l = master_volume;
    } else if(_balance > 0){
        volume_l = master_volume - ((master_volume*_balance)/50);
        volume_r = master_volume;
    } else {
        volume_l = master_volume;
        volume_r = master_volume;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Open_input
 * Input:       _input_selected: input selezionato
 *              
 * Overview:    Chiude il rel dedicato all'imput selezionato e invia volume + trim al PGA2320
/* ============================================================================================================================================ */
void Open_input(unsigned char _input_selected){

    INTCONbits.RBIE = 0;
        
    PGA23xx_send_register_value(0x00, 0x00);                                            // reimposto il volume per adattare input trim

    // reset di tutti gli input 
    REL_IN_3 = 1;
    Delay_ms(20);
    REL_IN_3 = 0;
    
    // apertura input selezionato
    switch (_input_selected){
        case 0:
            REL_IN_0 = 1;
            Delay_ms(20);
            REL_IN_0 = 0;
            break;
        case 1:
            REL_IN_1 = 1;
            Delay_ms(20);
            REL_IN_1 = 0;
            break;
        case 2:
            REL_IN_2 = 1;
            Delay_ms(20);
            REL_IN_2 = 0;
            break;            
    }
    
    Delay_ms(100);
   
    INTCONbits.RBIF = 0;
    INTCONbits.RBIE = 1;
    
    input_trim = EEPROM_read(EE_INPUT_TRIM_ADDRESS + _input_selected);    
    PGA23xx_send_register_value(volume_r + input_trim,volume_l + input_trim);           // reimposto il volume per adattare input trim
}

/* ============================================================================================================================================= /
 * FUNCTION:    Power
 * Input:       _on_off: ON/OFF
 *              
 * Overview:    Accende o spegne il pic (sleep) e tutti i dispositivi
/* ============================================================================================================================================ */
void Power(unsigned char _on_off){

    switch (_on_off){
        case OFF:
            #ifdef UART_DEBUG
            Send_log_uart(OFF);
            #endif
            
            flag_to_power_off = 0;
            mode = P_DOWN;  
            PGA23xx_send_register_value(0,0);                                           // PGA2320 in muto
            GLCD_clear(PIXEL_OFF);                                                      // cancello tutto da display
            GLCD_set_display(OFF);                                                      // spengo display
            
            // forzo disattivazione SPI
            SPI_disable();
            LATCbits.LATC3 = 0;                                                         // SPI
            LATCbits.LATC5 = 0;                                                               
                       
            #ifdef REL_HEADPHONE_INSTALLED
            REL_HEADPHONE = OFF;                                                        // disabilito uscita cuffie
            #endif
            
//            RELE_IO = RELE_IO & 0b11000000;                                           // disabilito tutti gli ingressi
            // reset di tutti gli input 
            REL_IN_3 = 1;
            Delay_ms(20);
            REL_IN_3 = 0;
            
            D_5V_EN = OFF;                                                              // spengo alimentazione digitale (che spegne a sua volta l'alimentazione analogica)
            
            OLED_CS = 0;                                                                // resetto tutte le uscite manualmente per ridurre i consumi in standby
            OLED_RST = 0;
            OLED_DC = 0;
            PGA23xx_CS = 0;
            PGA23xx_ZCEN = 0;                                                           
                     
            Delay_s(1);
            
            T2CONbits.TMR2ON = 0;                                                       // fermo timer per encoder
            T0CONbits.TMR0ON = 0;                                                       // fermo timer per anti bounce
            INTCON3bits.INT2IF = 0;                                                     
            INTCON3bits.INT2IE = 1;                                                     // abilito interrupt alla pressione del tasto encoder
            INTCON3bits.INT1IF = 0;                                                     
            INTCON3bits.INT1IE = 1;                                                     // abilito interrupt alla pressione del tasto input
            
            INTCONbits.RBIE = 0;                                                        // disabilito interrupt on change (over voltage), viene riabilitato alla prima apertura input
            
            Clear_ir_buffers();                                                         // Resetta buffer, comando ricevuto e riabilita ricezione
            
            Sleep();
            Nop();
            
            break;
        case ON:
            #ifdef UART_DEBUG
            Send_log_uart(ON);
            #endif

            INTCON3bits.INT2IF = 0;                                                     // disabilito interrupt pulsanti per accensione
            INTCON3bits.INT2IE = 0;  
            INTCON3bits.INT1IF = 0;                                                     
            INTCON3bits.INT1IE = 0; 

            flag_to_power_on = 0;
            mode = MAIN; 
            D_5V_EN = ON;                                                               // abilito alimentazione
            Delay_ms(200);
            GLCD_init();
            PGA23xx_enable();                                                           // PGA23xx abilitato e volume a 0
            Delay_ms(200);
            PGA23xx_zcen(zcen);
            Apply_balance(balance); 
            GLCD_set_contrast(contrast);
            Write_main_layout();
         
            T2CONbits.TMR2ON = 1;                                                       // abilito timer encoder
                        
            if (HP_CHK == 1){                                                           // se le cuffie sono connesse
                hp_status = CONNECTED;
                Write_input();
                Write_volume();
             
                Open_input(input_selected);
               
                #ifdef REL_HEADPHONE_INSTALLED
                REL_HEADPHONE = ON;                                                     // abilito uscita cuffie  
                #endif                                                             
            } else {
                hp_status = DISCONNECTED;
                GLCD_set_font(font_9x15);
                GLCD_write_string(LABEL_NOHP_X,LABEL_NOHP_Y,58,  labels[4], SAVESPACE_ON, PIXEL_ON);
                GLCD_set_font(font_dense_18x46);
                GLCD_write_float(VALUE_VOLUME_NEG_X,VALUE_VOLUME_Y,78,-99.9,2,1,ZERO_CLEANING_ON,SAVESPACE_ON,PIXEL_ON);
            }
                        
            Clear_ir_buffers();                                                         // Resetta buffer, comando ricevuto e riabilita ricezione
            
            // reset encoder per prevenire cambio di volume che capitava all'avvio o falsi over voltage
            encoder = 0;
            flag_over_voltage =0;
                        
            if (OV_CHK == 1)
            flag_over_voltage = 1;
            
            break;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Receive_ir
 *              
 * Overview:    Riceve il comando IR, salva il risultato in ir_temp[] salva lo stato ir_status
 * Note:        La funzione viene chiamata all'interno di un interrupt ad alto livello bloccando di fatto il programma fino a che non viene ricevuto tutto il dato
/* ============================================================================================================================================ */
void Receive_ir(void){
    unsigned char _pulse_counter = 0;
    unsigned char _period_counter = 0;
    unsigned char _ir_in_status = 0;
    unsigned char _counter = 0;
    unsigned char _limit = 1;
    
    // remote type 3 SIRC con piu di 44 bit
    if (remote_type == 3)                                                               
        _limit = 2;
    
    for (unsigned char _u = 0; _u < _limit; _u++){
        _pulse_counter = 0;
        _period_counter = 0;
        _ir_in_status = 0;
        
        do{
            _ir_in_status = IR_IN;
            _period_counter = 0;

            // ogni 50 us incremento contatore finch cambia lo stato di IR (nuovo dato) o finch il contatore arriva a 250 (fine trasmissione)
            do{
                Delay_us(50);
                _period_counter++;
            } while (IR_IN == _ir_in_status && _period_counter < 250);   

            // salvo il conteggio
            ir_received[_pulse_counter] = _period_counter;
            _pulse_counter++;

        }while((_pulse_counter < 4) || (_pulse_counter < 88 && _period_counter < 250)); // _pulse_counter < 3 perch sharp manda i primi 2 bit + lunghi di 250
        
        // se remote 2 e  la prima ricezione svuoto il buffer e ricevo una seconda volta
        // comandi oltre i 44 bit dove la prima parte  sempre uguale ad ogni comando
        if (remote_type == 3 && _u == 0){  
            for (unsigned char _i = 0; _i < 88; _i++)
                ir_received[_i] = 0; 
        }
    }
    
    // creo gli 11 byte dell'array dei comandi.
    // per ogni byte contenuto nel buffer di ricezione salvo uno 0 se il conteggio della durata dell'impulso  <22, salvo uno se > 22 e minore di 240    
    for (unsigned char _i = 0; _i < 11; _i++){
        for (unsigned char _c = 0; _c < 8; _c++){            
            if (ir_received[_counter] > 20 && ir_received[_counter] < 240)
                ir_temp[_i] |= 1 << (7 - _c);
            _counter++;
        }        
    }    
    
    // remote 4 (comandi che cambiano il terzo bit ad ogni pressione/rilascio RC5)
    // elimino i bit che cambiano in modo che siano sempre uguali
    if (remote_type == 4)   
        ir_temp[0] &= 0b11000111; 
    
    ir_status = IR_RECEIVED;                                                            // flag ir ricevuto
    
    // debug, salva in eeprom ricevuto
    #ifdef UART_DEBUG
    for (unsigned char _i = 0; _i < 11; _i++){
        EEPROM_write(ir_temp[_i], EE_IR_RECEIVED_ADDRESS + _i);
    }
    #endif
}

/* ============================================================================================================================================= /
 * FUNCTION:    Decode_ir
 *               Comando decodificato:
 *              - 1: Power
 *              - 2: Volume +
 *              - 3: Volume -
 *              - 4: Input
 *              - 5: Menu
 *              - 6: Exit
 *              - 20: Volume+ mantenuto premuto (sharp o telecomandi con comando ripetizione dedicato)
 *              - 30: Volume- mantenuto premuto (sharp o telecomandi con comando ripetizione dedicato)
 *
 * Overview:   Confronta il dato ricevuto con gli array contenenti i comandi salvati. salva in ir_command il comando decodificati.
/* ============================================================================================================================================ */
void Decode_ir (void){
    static unsigned char _prev = 0;    
    
    // comparazione array    
    if (Compare_array(ir_temp, ir_power))
        ir_command = 1;
    else if (Compare_array(ir_temp, ir_volume_p))
        ir_command = 2;
    else if (Compare_array(ir_temp, ir_volume_m))
        ir_command = 3;
    else if (Compare_array(ir_temp, ir_input))
        ir_command = 4;
    else if (Compare_array(ir_temp, ir_menu))
        ir_command = 5;
    else if (Compare_array(ir_temp, ir_exit))
        ir_command = 6;
    else if (Compare_array(ir_temp, ir_sharp_repeate)){
        if (_prev == 2 || _prev == 20)
            ir_command = 20;
        else if (_prev == 3 || _prev == 30)
            ir_command = 30;
    }
    
    if (remote_type == 1)
        _prev = ir_command;
    
    // delay per tutte le varianti IR che mandano lo stesso comando mantenendo premuto il tasto
    else {
        // nuovo pulsante premuto o timer scaduto
        if (ir_command != _prev || anti_bounce == 0) {
            if (ir_command != 2 && ir_command != 3)
                anti_bounce = MSEC_ANTIBOUNCE * 3;
            else 
                anti_bounce = MSEC_ANTIBOUNCE;
                
            TMR0 = TMR0_PRELOAD;                    
            T0CONbits.TMR0ON = ON;
            _prev = ir_command;
        } else {
            ir_command = 0;
        }
 
    }
    ir_status = IR_DECODED;     
    
    // debug salva in eeprom decoded
    #ifdef UART_DEBUG
    EEPROM_write(ir_command, EE_IR_DECODED_ADDRESS);
    
    if (ir_command != 20 && ir_command != 30)
        Send_log_uart(IR_RECEIVED);    
    #endif
}

/* ============================================================================================================================================= /
 * FUNCTION:    Clear_ir_buffers
 *
 * Overview:    Cancellazione forzata buffer di ricezione IR e ripristino interrupt allo stato iniziale
/* ============================================================================================================================================ */
void Clear_ir_buffers (void){
    for (unsigned char _i = 0; _i < 88; _i++)
        ir_received[_i] = 0; 
    
    for (unsigned char _i = 0; _i<11; _i++)
        ir_temp[_i] =0;
    
    // riabilito interrupt per ricezione nuovi comandi
    ir_status = IR_IDLE;  
    ir_command = 0;  
    INTCONbits.INT0IF = 0;
    INTCONbits.INT0IE = ON;
}

/* ============================================================================================================================================= /
 * FUNCTION:    Compare_array
 * Input:       array da comparare
 * Output:      - 0: gli array sono diversi
 *              - 1: gli array sono uguali 
 *
 * Overview:    Confronta 2 array da 11 byte, torna 1 se tutti i byte sono uguali, 0 se anche solo 1 byte  diverso.
/* ============================================================================================================================================ */
unsigned char Compare_array(unsigned char * _array_1, unsigned char * _array_2){
    
    for (unsigned char _i = 0; _i<11; _i++){
        if (_array_1[_i] != _array_2[_i])
            return 0;
    }
    
    return 1;
}

/* ============================================================================================================================================= /
 * FUNCTION:    Copy_array
 * Input:       _array_1 = array di salvataggio
 *              _array_2 = array da copiare
 *
 * Overview:    Copia 2 array da 11 byte
/* ============================================================================================================================================ */
void Copy_array(unsigned char * _array_1, unsigned char * _array_2){
    
    for (unsigned char _i = 0; _i<11; _i++)
        _array_1[_i] = _array_2[_i];
}

/* ============================================================================================================================================= /
 * FUNCTION:    Save_eeprom_default
 *              
 * Overview:    Salva i valori di default in eeprom (primo avvio o avvio con encoder e input premuti)
/* ============================================================================================================================================ */
void Save_eeprom_default(void){
    EEPROM_write(0x00, EE_VOLUME_ADDRESS);
    EEPROM_write(0x00, EE_BALANCE_ADDRESS);
    EEPROM_write(0x00, EE_INPUT_SEL_ADDRESS);
    EEPROM_write(PGA23xx_ZCEN_ON, EE_ZCEN_ADDRESS);
    EEPROM_write(0x20, EE_FIRST_START_ADDRESS);
    EEPROM_write(0x00, EE_INPUT_TRIM_ADDRESS);
    EEPROM_write(0x00, EE_INPUT_TRIM_ADDRESS + 1);
    EEPROM_write(0x00, EE_INPUT_TRIM_ADDRESS + 2);
    EEPROM_write(0x00, EE_INPUT_TRIM_ADDRESS + 3);
    EEPROM_write(0x00, EE_INPUT_TRIM_ADDRESS + 4);
    EEPROM_write(0x00, EE_INPUT_TRIM_ADDRESS + 5);
    EEPROM_write(0x7F, EE_CONTRAST_ADDRESS);
    EEPROM_write(0x01, EE_REMOTE_TYPE_ADDRESS);
    EEPROM_write(0x00, EE_INPUT_NAME_ADDRESS);
    EEPROM_write(0x01, EE_INPUT_NAME_ADDRESS+1);
    EEPROM_write(0x02, EE_INPUT_NAME_ADDRESS+2);
    EEPROM_write(0x00, EE_INPUT_NAME_ADDRESS+3);
    EEPROM_write(0x00, EE_INPUT_NAME_ADDRESS+4);
    EEPROM_write(0x00, EE_INPUT_NAME_ADDRESS+5);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Load_eeprom_variables
 *              
 * Overview:    Carica le variabili salvate in EEPROM nelle variabili globali
/* ============================================================================================================================================ */
void Load_eeprom_variables(void){
    master_volume = EEPROM_read(EE_VOLUME_ADDRESS);
    balance = EEPROM_read(EE_BALANCE_ADDRESS);
    input_selected = EEPROM_read(EE_INPUT_SEL_ADDRESS);
    input_trim = EEPROM_read(EE_INPUT_TRIM_ADDRESS + input_selected);
    zcen = EEPROM_read(EE_ZCEN_ADDRESS);
    contrast = EEPROM_read(EE_CONTRAST_ADDRESS);
    remote_type = EEPROM_read(EE_REMOTE_TYPE_ADDRESS);
    
    for (unsigned char _i = 0; _i < IO_USED; _i++){
        name_selected[_i] = EEPROM_read(EE_INPUT_NAME_ADDRESS + _i);       
    }
    
}

/* ============================================================================================================================================= /
 * FUNCTION:    Save_eeprom_remote
 *              
 * Overview:    Salva i comandi IR registrati in EEPROM
/* ============================================================================================================================================ */
void Save_eeprom_remote(void){    
    for (unsigned char _i = 0; _i < 11; _i++){
        EEPROM_write(ir_power[_i], EE_IR_POWER_ADDRESS + _i);
    }
    
    for (unsigned char _i = 0; _i < 11; _i++){
        EEPROM_write(ir_volume_p[_i], EE_IR_VOLUME_P_ADDRESS + _i);
    }
    
    for (unsigned char _i = 0; _i < 11; _i++){
        EEPROM_write(ir_volume_m[_i], EE_IR_VOLUME_M_ADDRESS + _i);
    }
    
    for (unsigned char _i = 0; _i < 11; _i++){
        EEPROM_write(ir_input[_i], EE_IR_INPUT_ADDRESS + _i);
    }
    
    for (unsigned char _i = 0; _i < 11; _i++){
        EEPROM_write(ir_menu[_i], EE_IR_MENU_ADDRESS + _i);
    }
    
    for (unsigned char _i = 0; _i < 11; _i++){
        EEPROM_write(ir_exit[_i], EE_IR_EXIT_ADDRESS + _i);
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Load_eeprom_remote
 *              
 * Overview:    Carica i comandi IR da EEPROM alle variabili globali
/* ============================================================================================================================================ */
void Load_eeprom_remote(void){     
    for (unsigned char _i = 0; _i < 11; _i++){
        ir_power[_i] = EEPROM_read(EE_IR_POWER_ADDRESS + _i);
    }
    
    for (unsigned char _i = 0; _i < 11; _i++){
        ir_volume_p[_i] = EEPROM_read(EE_IR_VOLUME_P_ADDRESS + _i);
    }
    
    for (unsigned char _i = 0; _i < 11; _i++){
        ir_volume_m[_i] = EEPROM_read(EE_IR_VOLUME_M_ADDRESS + _i);
    }
    
    for (unsigned char _i = 0; _i < 11; _i++){
        ir_input[_i] = EEPROM_read(EE_IR_INPUT_ADDRESS + _i);
    }
    
    for (unsigned char _i = 0; _i < 11; _i++){
        ir_menu[_i] = EEPROM_read(EE_IR_MENU_ADDRESS + _i);
    }
    
    for (unsigned char _i = 0; _i < 11; _i++){
        ir_exit[_i] = EEPROM_read(EE_IR_EXIT_ADDRESS + _i);
    }
}        

/* ============================================================================================================================================= /
 * CICLI PER DEBUG VIA UART
/* ============================================================================================================================================ */

#ifdef UART_DEBUG
/* ============================================================================================================================================= /
 * FUNCTION:    Send_log_uart
 * Input:       _x: porzioni di dati da inviare a seconda del punto di esecuzione (BOOT/POWER ON/POWER OFF)
 *              
 * Overview:    Invia i dati di debug via UART
/* ============================================================================================================================================ */
void Send_log_uart(unsigned char _x){

    unsigned char _temp = 0;
    unsigned char _string[4] = {0};
    
    _string[3] = 0x00;
    
    if (_x == BOOT){
        USART2_send_string("Headphone Amp Debug\r\n");
        USART2_send_string("HW rev: ");
        USART2_send_string(PCB_VERS);
        USART2_send_string("\r\nFW rev: ");
        USART2_send_string(FW_VERS);
        USART2_send_string("\r\n\r\nAvvio dopo connessione AC");
    }else if (_x == ON){
        USART2_send_string("\r\n\r\nAccensione");
    } else if (_x == OFF){
        USART2_send_string("\r\n\r\nSpegnimento");
    }else if (_x == WAKE){
        USART2_send_string("\r\n\r\nUscita da sleep per check pulsanti e IR");
    }else if (_x == IR_RECEIVED)
        USART2_send_string("\r\n\r\nIR ricevuto");
    
    if (_x == BOOT || _x == ON){
        USART2_send_string("\r\n\r\nRCON register: ");
        for (unsigned char _i=0; _i<8; _i++){
            _temp = (RCON >> 7 -_i) & 0b0000001;
            _temp += '0';
            USART2_send_char(_temp);   
        }
        
        USART2_send_string("\r\nVolume: ");
        Convert_2_string(master_volume, _string);
        USART2_send_string(_string);
                    
        USART2_send_string("\r\nBalance: ");
        Convert_2_string(balance, _string);
        USART2_send_string(_string);
        
        USART2_send_string("\r\nInput selezionato: ");
        USART2_send_char(input_selected + '0');
        
        USART2_send_string("\r\nZCEN: ");
        _temp = (unsigned char) zcen + '0';
        USART2_send_char(_temp);
        
        for (unsigned char _i = 0; _i < IO_USED; _i++) {
            USART2_send_string("\r\nInput trim ");
            USART2_send_char(_i + '0');
            USART2_send_string(": ");
            Convert_2_string(EEPROM_read(EE_INPUT_TRIM_ADDRESS + _i), _string);
            USART2_send_string(_string);
        }
        
        USART2_send_string("\r\nContrasto: ");
        Convert_2_string(contrast, _string);
        USART2_send_string(_string);
        
        USART2_send_string("\r\nRemote type: ");
        USART2_send_char(remote_type + '0');
        
        USART2_send_string("\r\nComandi IR registrati: ");
        
        _string[2] = 0x00;
        
        for (unsigned char _a = 0; _a < 6; _a++){
            switch (_a+1){
            case 1:
                USART2_send_string("\r\nPOWER:    ");
                for (unsigned char _b = 0; _b < 11; _b++){
                    Convert_2_string_hex(ir_power[_b], _string);
                    USART2_send_string(_string);
                    USART2_send_char(' ');
                }
                break;
            case 2:
                USART2_send_string("\r\nVOLUME +: ");
                for (unsigned char _b = 0; _b < 11; _b++){
                    Convert_2_string_hex(ir_volume_p[_b], _string);
                    USART2_send_string(_string);
                    USART2_send_char(' ');
                }
                break;
            case 3:
                USART2_send_string("\r\nVOLUME -: ");
                for (unsigned char _b = 0; _b < 11; _b++){
                    Convert_2_string_hex(ir_volume_m[_b], _string);
                    USART2_send_string(_string);
                    USART2_send_char(' ');
                }
                break;
            case 4:
                USART2_send_string("\r\nINPUT:    ");
                for (unsigned char _b = 0; _b < 11; _b++){
                    Convert_2_string_hex(ir_input[_b], _string);
                    USART2_send_string(_string);
                    USART2_send_char(' ');
                }
                break;
            case 5:
                USART2_send_string("\r\nMENU:     ");
                for (unsigned char _b = 0; _b < 11; _b++){
                    Convert_2_string_hex(ir_menu[_b], _string);
                    USART2_send_string(_string);
                    USART2_send_char(' ');
                }
                break;
            case 6:
                USART2_send_string("\r\nEXIT:     ");
                for (unsigned char _b = 0; _b < 11; _b++){
                    Convert_2_string_hex(ir_exit[_b], _string);
                    USART2_send_string(_string);
                    USART2_send_char(' ');
                }
                break;
            }
        }
    }
        
    if (_x == BOOT || _x == ON || _x == OFF || _x == IR_RECEIVED){
        USART2_send_string("\r\nUltimo comando IR ricevuto (salvato in fase di ricezione): ");
        for (unsigned char _b = 0; _b < 11; _b++){
            Convert_2_string_hex(EEPROM_read(EE_IR_RECEIVED_ADDRESS +_b), _string);
            USART2_send_string(_string);
            USART2_send_char(' ');
        }

        USART2_send_string("\r\nUltimo comando IR decodificato (salvato in fase di ricezione): ");
        switch (EEPROM_read(EE_IR_DECODED_ADDRESS)){
            case 0:
                USART2_send_string("Nessun comando decodificato");
                break;    
            case 1:
                USART2_send_string("POWER");
                break;
            case 2:
                USART2_send_string("VOLUME +");
                break;
            case 3:
                USART2_send_string("VOLUME -");
                break;
            case 4:
                USART2_send_string("INPUT");
                break;
            case 5:
                USART2_send_string("MENU");
                break;
            case 6:
                USART2_send_string("EXIT");
                break;
            case 20:
                USART2_send_string("VOLUME + MANTENUTO PREMUTO");
                break;
            case 30:
                USART2_send_string("VOLUME - MANTENUTO PREMUTO");
                break;
        }      
    }

    if (_x == BOOT || _x == ON || _x == OFF){
        USART2_send_string("\r\nComando IR attuale: ");
        switch (ir_command){
            case 0:
                USART2_send_string("Nessun comando decodificato");
                break;    
            case 1:
                USART2_send_string("POWER");
                break;
            case 2:
                USART2_send_string("VOLUME +");
                break;
            case 3:
                USART2_send_string("VOLUME -");
                break;
            case 4:
                USART2_send_string("INPUT");
                break;
            case 5:
                USART2_send_string("MENU");
                break;
            case 6:
                USART2_send_string("EXIT");
                break;
            case 20:
                USART2_send_string("VOLUME + MANTENUTO PREMUTO");
                break;
            case 30:
                USART2_send_string("VOLUME - MANTENUTO PREMUTO");
                break;
        }   

        USART2_send_string("\r\nStato IR attuale: ");
        switch (ir_status){
            case IR_IDLE:
                USART2_send_string("IDLE");
                break;
            case IR_INCOMING:
                USART2_send_string("IR IN ARRIVO");
                break;
            case IR_RECEIVED:
                USART2_send_string("IR RICEVUTO");
                break;
            case IR_DECODED:
                USART2_send_string("IR DECODIFICATO");
                break;
        }

        USART2_send_string("\r\nFlag to_power_on: ");
        USART2_send_char(flag_to_power_on + '0');

        USART2_send_string("\r\nFlag to_power_off: ");
        USART2_send_char(flag_to_power_off + '0');

        USART2_send_string("\r\nModalita attuale: ");
        switch (mode){
            case MAIN:
                USART2_send_string("IN FUNZIONE");
                break;    
            case P_DOWN:
                USART2_send_string("IN SLEEP");
                break;    
            case SETUP:
                USART2_send_string("IN CICLO SETUP");
                break;    
        }

        USART2_send_string("\r\nEncoder attuale: ");
        if (encoder < 0){
            _temp = ~ encoder +1;
            USART2_send_char('-');
        } else
            _temp = encoder;

        Convert_2_string(_temp, _string);
        USART2_send_string(_string);

        USART2_send_string("\r\nFlag over voltage: ");
        USART2_send_char(flag_over_voltage + '0');  
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Convert_2_string
 * Input:       _value: valore da convertire
 *              _pointer: puntatore a locazione di salvataggio stringa
 *              
 * Overview:    Converte un byte in una stringa di valori decimali
/* ============================================================================================================================================ */
void Convert_2_string (unsigned char _value, unsigned char * _pointer){
    
    unsigned char _temp = 0;
    
    _pointer[0] = (_value / 100) +'0';
    _temp = _value - (100*(_pointer[0]-'0'));
    _pointer[1] = (_temp / 10) +'0';
    _pointer[2] = (_temp % 10) +'0';
}

/* ============================================================================================================================================= /
 * FUNCTION:    Convert_2_string_hex
 * Input:       _value: valore da convertire
 *              _pointer: puntatore a locazione di salvataggio stringa
 *              
 * Overview:    Converte un byte in una stringa di valori esadecimali
/* ============================================================================================================================================ */
void Convert_2_string_hex (unsigned char _value, unsigned char * _pointer){
    unsigned char _temp = 0;    
    
    
    _temp = (_value >> 4) & 0x0F;
    if (_temp < 0x0A)
        _pointer[0] = _temp + '0';
    else 
        _pointer[0] = _temp + 0x37;   
    
    _temp = _value & 0x0F;
    if (_temp < 0x0A)
        _pointer[1] = _temp + '0';
    else 
        _pointer[1] = _temp + 0x37;
}

#endif
