/* ============================================================================================================================================= /
 * LIBRARY:         Gestione OLED SSD1306/SH1106
 * FILE:            OLED_SSD1306.c
 * LAST REV:        1.1
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         21/08/2017
 * UPDATED:         10/10/2017
 *
 * DESCRIP:         Gestione di display GLCD OLED SSD1306/SH1106 128*64
 *
 * FUNCTIONS:       PRIVATE:
 *                  GLCD_send_command
 *                  GLCD_send_data(_data);
 *                  GLCD_read_data ();
 *                  GLCD_set_pointer(_page,_column);
 *                  GLCD_set_addressing_mode(_mode);
 *                  GLCD_set_display(_status);
 *                  
 *                  PUBBLIC:
 *                  GLCD_init();
 *                  GLCD_disable();
 *                  GLCD_set_contrast(_contrast);
 *                  GLCD_negative_write(_on_off);
 *                  GLCD_scroll_set(_scrolling, _page_start, _page_end, _time);
 *                  GLCD_scroll_en(_on_off);
 *                  GLCD_clear(_on_off);
 *                  GLCD_draw_pixel(_x, _y, _on_off);
 *                  GLCD_write_byte(_page, _x, _data, _preserve_old, _on_off);
 *                  GLCD_draw_image(* _image);
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.1               10/10/2017          - Aggiunta compatibilit con SH1106
  1.0               26/08/2017          - Prima build completa
//! [History]
/* ============================================================================================================================================ */

#ifndef OLED_SSD1306
#define OLED_SSD1306

/* INCLUDES =================================================================================================================================== */
#include    "OLED_SSD1306_settings.h"
#include    "OLED_SSD1306.h"
#include    "GLCD_common_graphics.c"

#ifdef OLED_MODE_I2C
unsigned char oled_i2c_dev_addr = 0;
/*
 * Utilizzo la variabile in questo modo altrimenti ogni funzione dovrebbe avere come parametro in ingresso l'indirizzo del display usato
 * ma visto che in SPI non viene utilizzato e visto che la libreria GLCD_common_graphics.c  in comune con altri display che non utilizzano il 
 * protocollo usb sarebbero tutte variabili inutili da passare ad ogni signola funzione di scrittura del display 
 */
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_send_command
 * Input:       _command: comando da inviare (unsigned char)
 *
 * Overview:    Invia i byte di comando al display
/* ============================================================================================================================================ */
void GLCD_send_command (unsigned char _command){
#ifdef OLED_MODE_SPI_4
    OLED_CS = LOW;
    OLED_DC = LOW;
    _OLED_SPI_send_byte(_command);
    OLED_CS = HIGH;
#elif defined OLED_MODE_I2C
    _OLED_I2C_1ByteAddr_1ByteData_write(OLED_I2C_ADDR | (oled_i2c_dev_addr << 1), OLED_I2C_COMMAND, _command); 
#endif
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_send_data
 * Input:       _data: dato da scrivere su display (unsigned char)
 *
 * Overview:    Invia i byte da scrivere in GRAM al display
/* ============================================================================================================================================ */
void GLCD_send_data (unsigned char _data){
#ifdef OLED_MODE_SPI_4
    OLED_CS = LOW;
    OLED_DC = HIGH;
    _OLED_SPI_send_byte(_data);
    OLED_CS = HIGH;
#elif defined OLED_MODE_I2C
     _OLED_I2C_1ByteAddr_1ByteData_write(OLED_I2C_ADDR | (oled_i2c_dev_addr << 1), OLED_I2C_DATA, _data); 
#endif
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_read_data
 * Output:      Dato letto dal display
 * 
 * Overview:    Legge il dato scritto alla pagina/colonna precedentemente indirizzate
 * Nota:        Solo per SH1106 in comunicazione I2C
/* ============================================================================================================================================ */
#if defined OLED_DRIVER_SH1106 && defined OLED_MODE_I2C
unsigned char GLCD_read_data (void){
    return _OLED_I2C_1ByteAddr_1ByteData_read(OLED_I2C_ADDR | (oled_i2c_dev_addr << 1),OLED_I2C_DATA);
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_set_display
 * Input:       _status: ON o OFF (unsigned char)
 *
 * Overview:    Attiva o disattiva il display
/* ============================================================================================================================================ */
void GLCD_set_display(unsigned char _status) {

    if (_status == ON) {                                                                // accendo
        GLCD_send_command(OLED_ON);                                                     // 0xAF
        Delay_ms(110);
    } else {                                                                            // spengo
        GLCD_send_command(OLED_OFF);                                                    // 0xAE
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_init
 *
 * Overview:    Inizializzazione display
 * Precond:     Definire l'orientamento del display in GLCD_SSD1306_settings.h
/* ============================================================================================================================================ */
void GLCD_init(void){
    #ifdef OLED_MODE_SPI_4                                                              // impostazione BUS SPI se SPI selezionato, I2C va impostato globalmente in main
    _OLED_SPI_disable();
    _OLED_SPI_enable(SAMPLE_AT_MIDDLE, TRA_IDLE_TO_ACTIVE, IDLE_IS_HIGH, OLED_SPI_CLOCK);
    OLED_CS = HIGH; 
    #endif

    OLED_RST = LOW;
    Delay_ms(50);
    OLED_RST = HIGH;
    Delay_ms(100);
    
    GLCD_send_command(OLED_OFF);                                                        // 0xAE    
    GLCD_send_command(OLED_SET_CONTRAST);                                               // 0x81
    GLCD_send_command(0x7F);
    GLCD_send_command(OLED_ALL_PIXELL_RESUME_RAM);                                      // 0xA4 PIXEL RAM CONTENT
    GLCD_send_command(OLED_SET_NORMAL);                                                 // 0xA6 NORMAL DISPLAY
    GLCD_send_command(OLED_ADDRESSING_MODE_REG);                                        // 0x20 addressing mode
    GLCD_send_command(OLED_ADDRESSING_PAGE); 
    GLCD_send_command(0x00);                                                            // set lower column
    GLCD_send_command(0x10);                                                            // set higher column
    GLCD_send_command(0xB0);                                                            // set page start
    GLCD_send_command(OLED_SET_START_LINE | 0x00);                                      // 0x40 START LINE 0
    #ifdef OLED_PORTRAIT
        GLCD_send_command(OLED_SEGMENT_REMAP_OFF);                                      // 0xA0 SEGMENT REMAP (DA DEFINIRE IN BASE ALL'ORIENTAMENTO DEL DISPLAY)
        GLCD_send_command(OLED_COM_SCAN_DIR_NOR);                                       // 0xC0 COM SCAN DIRECTION (DA DEFINIRE IN BASE ALL'ORIENTAMENTO DEL DISPLAY)
    #elif defined OLED_PORTRAIT_180
        GLCD_send_command(OLED_SEGMENT_REMAP_ON);                                       // 0xA1 SEGMENT REMAP (DA DEFINIRE IN BASE ALL'ORIENTAMENTO DEL DISPLAY)
        GLCD_send_command(OLED_COM_SCAN_DIR_INV);                                       // 0xC8 COM SCAN DIRECTION (DA DEFINIRE IN BASE ALL'ORIENTAMENTO DEL DISPLAY)
    #endif
    GLCD_send_command(OLED_SET_MUX_RATIO);                                              // 0xA8
    GLCD_send_command(0x3F);                                                            // MUX RATIO = 64 MUX
    GLCD_send_command(OLED_SET_DISP_OFFSET);                                            // 0xD3
    GLCD_send_command(0x00);                                                            // DISPLAY OFFSET = 0
    GLCD_send_command(OLED_COM_HW_CONFIG);                                              // 0xDA
    GLCD_send_command(0x12);
    GLCD_send_command(OLED_SET_DISP_CLOCK);                                             // 0xD5
    GLCD_send_command(0x80);
    GLCD_send_command(OLED_SET_PRECH_PERIOD);                                           //0xD9
    GLCD_send_command(0xF1);
    GLCD_send_command(OLED_SET_VCOM_DETECT);                                            // 0xDB
    GLCD_send_command(0x40);  
    GLCD_send_command(OLED_CHARGE_PUMP_REG);                                            // 0x8D
    GLCD_send_command(0x14);
    GLCD_clear(PIXEL_OFF);
    GLCD_send_command(OLED_ON);                                                         // DISPLAY ON
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_disable
 *
 * Overview:    Disabilita il modulo SPI
 * ============================================================================================================================================ */
void GLCD_disable(void){
    #ifdef OLED_MODE_SPI_4
    _OLED_SPI_disable();
    OLED_CS = HIGH;  
    #endif         
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_set_contrast
 * Input:       _contrast: valore del contrasto 0~255 (unsigned char)
 *
 * Overview:    Imposta il contrasto del display
 * ============================================================================================================================================ */
void GLCD_set_contrast(unsigned char _contrast){
    GLCD_send_command(OLED_SET_CONTRAST);
    GLCD_send_command(_contrast);
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_negative_write
 * Input:       _on_off: ON: scritte nere sfondo bianco, OFF scritte bianche sfondo nero (unsigned char)
 *
 * Overview:    Abilita o disabilita la scrittura in negativo 
 * ============================================================================================================================================ */
void GLCD_negative_write(unsigned char _on_off){
    if (_on_off == ON)
        GLCD_send_command(OLED_SET_INVERSE);                                            // 0xA7 INVERSE DISPLAY
    else
        GLCD_send_command(OLED_SET_NORMAL);                                             // 0xA6 NORMAL DISPLAY
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_set_pointer
 * Input:       _page: valore della pagina (0 ~ MAX_PAGES) (unsigned char)
 *              _column: valore della colonna (0 ~ X_RESOLUTION) (unsigned char)
 *
 * Overview:    Imposta il puntatore GRAM alla pagina e colonna desiderate
 * ============================================================================================================================================ */
void GLCD_set_pointer(unsigned char _page, unsigned char _column){  
    GLCD_send_command(0xB0 | (_page & 0b00000111));                                     // pagina
    GLCD_send_command(_column & 0x0F);                                                  // colonna byte basso
    GLCD_send_command(((_column >> 4) & 0x0F)|0x10);                                    // colonna byte alto
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_set_addressing_mode
 * Input:       _mode: modalit di indirizzamento (unsigned char):
 *                      OLED_ADDRESSING_HORIZONTAL: ad ogni byte scritto in GRAM viene incrementato in automatico il valore della colonna, raggiunto il valore massimo di colonne viene incrementato il valore della pagina e portato a zero il valore delle colonne
 *                      OLED_ADDRESSING_VERTICAL: ad ogni byte scritto in GRAM viene incrementato in automatico il valore della pagina, raggiunto il valore massimo di pagine viene incrementato il valore della colonna e portato a zero il valore delle pagine
 *                      OLED_ADDRESSING_PAGE: ad ogni byte scritto in GRAM viene incrementato in automatico il valore della colonna, raggiunto il valore massimo di colonne viene portato a zero il valore delle colonne ma il valore della pagina resta invariato
 *
 * Overview:    Imposta la modalit di scrittura del display
 * Note:        Solo per SSD1306
 * ============================================================================================================================================ */
#ifdef OLED_DRIVER_SSD1306
void GLCD_set_addressing_mode(unsigned char _mode){
    GLCD_send_command(OLED_ADDRESSING_MODE_REG);
    GLCD_send_command(_mode); 
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_scroll_set
 * Input:       _scrolling: modalit di scrolling (unsigned char): OLED_HORIZONTAL_SCROLL_RIGHT/OLED_HORIZONTAL_SCROLL_LEFT/OLED_VERTICAL_HORIZONTAL_RIGHT/OLED_VERTICAL_HORIZONTAL_LEFT
 *              _page_start: pagina di inizio scrolling 0~7 (unsigned char)
 *              _page_end: pagina di fine scrolling 0~7 (unsigned char)
 *              _time: tempo di scrolling 1~7 (unsigned char)
 *
 * Overview:    Imposta la modalit di scrolling del display
 * Note:        Per lo scrolling verticale tutto il display verr spostato in alto ma solo le pagine selezionate verranno spostate anche a sinistra o destra.            
 *              Per spostare solo l'intero display in alto selezionare una pagina vuota 
 *              Solo per SSD1306
 * ============================================================================================================================================ */
#ifdef OLED_DRIVER_SSD1306
void GLCD_scroll_set(unsigned char _scrolling, unsigned char _page_start, unsigned char _page_end, unsigned char _time){
    
    GLCD_scroll_en(OLED_SCROLL_DISABLED);
    
    GLCD_send_command(_scrolling);
    GLCD_send_command(0x00);                                                            // dummy byte
    GLCD_send_command(_page_start & 0x07);    
    GLCD_send_command(_time & 0x07);
    GLCD_send_command(_page_end & 0x07);
    
    if (_scrolling == OLED_HORIZONTAL_SCROLL_RIGHT || _scrolling == OLED_HORIZONTAL_SCROLL_LEFT){
        GLCD_send_command(0x00);
        GLCD_send_command(0xFF);
    } else {
        GLCD_send_command(0x01);                                                        // vertical scrolling offset
    }
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_scroll_en
 * Input:       _on_off: OLED_SCROLL_ENABLED/OLED_SCROLL_DISABLED (unsigned char)
 *
 * Overview:    Attiva o disattiva lo scrolling con i parametri impostati tramite funzione GLCD_set_scroll()
 * Precond:     Impostare il tipo di scrolling tramite funzione GLCD_set_scroll();
 * Postcond:    Dopo aver disattivato lo scrolling il display va riscritto.
 * Note:        Con scrolling attivo non  possibile scrivere nuovi valori sul display e cambiare i parametri di scroll
 *              Solo per SSD1306
 * ============================================================================================================================================ */
#ifdef OLED_DRIVER_SSD1306
void GLCD_scroll_en(unsigned char _on_off){
    GLCD_send_command(_on_off);
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_clear
 * Input:       _on_off: PIXEL_ON/PIXEL_OFF (unsigned char)
 *
 * Overview:    Cancella o riempie tutto il display
 * ============================================================================================================================================ */
void GLCD_clear(unsigned char _on_off){
    
    #ifdef OLED_DRIVER_SSD1306
    GLCD_set_addressing_mode(OLED_ADDRESSING_HORIZONTAL);                               // indirizzamento orizzontale per scrivere in automatico tutto il display
    GLCD_set_pointer(0,0);
    #endif  
    
    for (unsigned char _page = 0; _page < MAX_PAGES; _page++){
        #ifdef OLED_DRIVER_SH1106                                                       // SH1106 ha solo indirizzamento per pagina,
        GLCD_set_pointer(_page, SH1106_X_OFFSET);                                       // va quindi impostata la pagina manualmente, la colonna si incrementa da sola ad ogni scrittura
        #endif  
        for (unsigned char _x = 0; _x < X_RESOLUTION; _x++){
            GLCD_send_data(_on_off);
        }
    }
    
    #ifdef OLED_DRIVER_SSD1306
    GLCD_set_addressing_mode(OLED_ADDRESSING_PAGE);
    #endif
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_draw_pixel
 * Input:       _x: indirizzo x: 1~X_RESOLUTION (unsigned char)
 *              _y: indirizzo y: 1~Y_RESOLUTION (unsigned char)
 *              _on_off: bit acceso o spento: PIXEL_ON o PIXEL_OFF (unsigned char)
 *
 * Overview:    Scrive un bit nell'indirizzo xy selezionato
 * Note:        In scrittura SPI non  possibile leggere il display, tutti i bit della pagina verrano scritti cancellando di fatto precedenti valori.
 *              Se vengono scritti in successione pi pixel nella stessa pagina e nella stessa colonna vengono riscritti sempre tutti grazie alla variabile
 *              _buffer che di fatto contiene il valore dell'intera pagina/colonna
/* ============================================================================================================================================ */
void GLCD_draw_pixel(unsigned char _x, unsigned char _y, unsigned char _on_off){
    unsigned char _page = 0;
    static unsigned char _prev_page = 0;
    static unsigned char _prev_x = 0;
    static unsigned char _buffer;
    
    _x--;                                                                               // riporto i valori a 0 ~ 127
    _y--;                                                                               // riporto i valori a 0 ~ 63
    
    if (_x < X_RESOLUTION && _y < Y_RESOLUTION){

        #ifdef OLED_DRIVER_SH1106                                                       // offset per SH1106
        _x += SH1106_X_OFFSET;
        #endif

        _page = _y >> 3;                                                                // /8, calcolo in quale pagina cade il pixel
        
        
        /* In comunicazione SPI e per SSD1306 anche I2C tengo un buffer per pagina perch non posso 
           rileggere i dati dal display
        */
    #if defined OLED_DRIVER_SSD1306 || defined OLED_MODE_SPI_4                                   
        if (_page != _prev_page || _x != _prev_x)
            _buffer = 0;
        
        _buffer |= _on_off << (_y & 0b00000111);
        
        GLCD_set_pointer(_page, _x);                                                    // puntatore a pagina e colonna
        GLCD_send_data(_buffer);                                                        // invio byte pagina contenente il singolo pixel
        _prev_page = _page;
        _prev_x = _x;
        
        /* Per SH1106 in comunicazione I2C rileggo il byte e aggiungo o tolgo il bit del pixel attuale
        */
    #elif defined OLED_DRIVER_SH1106 && defined OLED_MODE_I2C
        unsigned char _shift = 0;
        unsigned char _bit = 0;
        unsigned char _readed_data = 0;
        
        GLCD_set_pointer(_page, _x);                                                    // puntatore a pagina e colonna
        _readed_data = GLCD_read_data();                                                // dummy read
        _readed_data = GLCD_read_data();

        _shift = (_y & 0b00000111);                                                     // controllo quale bit devo modificare
        _bit = 0b00000001 << _shift;

        if (_on_off == ON)                                                              // se devo accendere il pixel
            _buffer = _readed_data | _bit;                                              // ricavo il nuovo valore del byte aggiungendo il bit del pixel da accendere
        else
            _buffer = _readed_data & (~ _bit);                                          // altrimenti togliendo il bit del pixel da spegnere

        GLCD_set_pointer(_page, _x);                                                    // puntatore a pagina e colonna
        GLCD_send_data(_buffer);                                                        // mando il nuovo byte da scrivere
#endif  
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_byte
 * Input:       _page: pagina da selezionare: 0~7 (unsigned char)
 *              _address_x: indirizzo x: 0~127 (unsigned char)
 *              _data: byte da scrivere (unsigned char)
 *              _preserve_old: PRESERVE/OVERWRITE (unsigned char) Preserva o sovrascrive i dati sulla stessa pagina
 *              _on_off: PIXEL_ON/PIXEL_OFF (unsigned char)
 *              
 * Overview:    Scrive un byte nella pagina selezionata all'indirizzo x selezionato
 * Note:        Utilizzato in funzioni di scrittura font e immagini
 *              In scrittura SPI non  possibile leggere il display, tutti i bit della pagina verrano scritti cancellando di fatto precedenti valori
/* ============================================================================================================================================ */
void GLCD_write_byte(unsigned char _page, unsigned char _x, unsigned char _data, unsigned char _preserve_old, unsigned char _on_off){
    if (_on_off == OFF)
        _data = ~_data;
    
    if (_x < X_RESOLUTION && _page < MAX_PAGES) {                                       // se x e y sono nei limiti di risoluzione
        
        #ifdef OLED_DRIVER_SH1106                                                       // offset per SH1106
        _x += SH1106_X_OFFSET;
        #endif

        /* Per SH1106 in comunicazione I2C rileggo il byte e aggiungo o tolgo il nuovo dato da scrivere
        */
    #if defined OLED_DRIVER_SH1106 && defined OLED_MODE_I2C
        unsigned char _readed_data = 0;
        
        if (_preserve_old == PRESERVE){
            GLCD_set_pointer(_page, _x);
            
            _readed_data = GLCD_read_data();                                            // dummy read
            _readed_data = GLCD_read_data();
    
            if (_on_off == ON){
                _data |= _readed_data;                                                  // modifico il dato da inviare in modo da non sovrascrivere
            } else {
                _data ^= _readed_data;
            }
        }
    #endif
        GLCD_set_pointer(_page, _x);
        GLCD_send_data (_data);
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_draw_image
 * Input:       _image: puntatore al primo indirizzo in flash dell'array contenente l'immagine
 *
 * Overview:    Scrive un'immagine salvata in array sul display
 * Nota:        Immagini convertite con LCD assistant, codifica byte verticale
/* ============================================================================================================================================ */
void GLCD_draw_image(const unsigned char * _image){
    
    #ifdef OLED_DRIVER_SSD1306
    GLCD_set_addressing_mode(OLED_ADDRESSING_HORIZONTAL);                               // indirizzamento orizzontale per scrivere in automatico tutto il display
    GLCD_set_pointer(0,0);
    #endif  
    
    for (unsigned char _page = 0; _page < MAX_PAGES; _page++){
        #ifdef OLED_DRIVER_SH1106                                                       // SH1106 ha solo indirizzamento per pagina,
        GLCD_set_pointer(_page, SH1106_X_OFFSET);                                       // va quindi impostata la pagina manualmente
        #endif  
        for (unsigned char _x = 0; _x < X_RESOLUTION; _x++){
            GLCD_send_data(*_image);
            _image++;
        }
    }
    #ifdef OLED_DRIVER_SSD1306
    GLCD_set_addressing_mode(OLED_ADDRESSING_PAGE);
    #endif  
}
#endif