/* ============================================================================================================================================= /
 * LIBRARY:         Gestione funzioni grafiche per GLCD monocromatici
 * FILE:            GLCD_common_graphics.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         26/09/2017
 * UPDATED:         
 *
 * DESCRIP:         Funzioni grafiche per librerie GLCD monocromatici
 *
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.0               26/09/2017          - Prima build 
//! [History]
/* ============================================================================================================================================ */

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file GLCD_common_graphics.c
 * Libreria di funzioni grafiche per display grafici monocromatici
 * - Per un corretto funzionamento:
 *  + Nessuna azione necessaria, la libreria  inclusa in automatico con la libreria specifica per il display grafico in uso
 */

 /**
 * Disegna una linea orizzontale su GLCD da punto _x_origin a punto _x_dest compresi su riga _y
 * @param   _x_start: coordinata x di partenza: 1~128 (unsigned char)
 * @param   _x_end: coordinata x finale: 1~128 (unsigned char)
 * @param   _y: coordinata y: 1~64 (unsigned char)
 * @param   _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 * @return  void
 */
void GLCD_draw_line_x(unsigned char _x_start, unsigned char _x_dest, unsigned char _y, unsigned char _on_off);

/**
 * Disegna una linea verticale su GLCD da punto _y_origin a punto _y_dest compresi su colonna _x
 * @param   _y_start: coordinata y di partenza: 1~64 (unsigned char)
 * @param   _y_end: coordinata y finale: 1~64 (unsigned char)
 * @param   _x: coordinata x: 1~128 (unsigned char)
 * @param   _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 * @return  void
 */
void GLCD_draw_y(unsigned char _y_origin, unsigned char _y_dest, unsigned char _x, unsigned char _on_off);

/**
 * Disegna una linea da punto a punto su GLCD da punto _x/_y_origin a punto x/_y_dest compresi
 * @param   _x_start: coordinata x di partenza: 1~128 (unsigned char)
 * @param   _y_start:  coordinata y di partenza: 1~64 (unsigned char)
 * @param   _x_end: coordinata x finale: 1~128 (unsigned char)
 * @param   _y_end: coordinata y finale: 1~64 (unsigned char)
 * @param   _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 * @return void
 */
void GLCD_draw_line_p2p (unsigned char _x_start, unsigned char _y_start, unsigned char _x_end, unsigned char _y_end, unsigned char _on_off);

/**
 * Disegna una linea da punto origine di lunghezza stabilita con angolo su GLCD
 * @param   _x_start: coordinata x di partenza: 1~128 (unsigned char)
 * @param   _y_start: coordinata y di partenza: 1-64 (unsigned char)
 * @param   _length: lunghezza: 1-128 (unsigned char)
 * @param   _angle: angolo: 0-360 (unsigned int)
 * @param   _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 * @return  void
 */
void GLCD_draw_line_angle(unsigned char _x_start, unsigned char _y_start, unsigned char _length,  unsigned int _angle, unsigned char _on_off);

/**
 * Disegna una rettangolo su GLCD
 * @param   _x_start: coordinata x di partenza: 1~128 (unsigned char)
 * @param   _y_start: coordinata y di partenza: 1~64 (unsigned char)
 * @param   _l: lunghezza: 1~128 (unsigned char)
 * @param   _h: altezza: 1~64 (unsigned char)
 * @param   _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 * @return void
 */
void GLCD_draw_rect(unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _on_off);

/**
 * Disegna un rettangolo pieno su GLCD compreso tra i punti x y indicati (punti compresi)
 * @param   _x_start: coordinata x di partenza: 1~128 (unsigned char)
 * @param   _y_start: coordinata y di partenza: 1~64 (unsigned char)
 * @param   _l: lunghezza: 1~128 (unsigned char)
 * @param   _h: altezza: 1~64 (unsigned char)
 * @param   _on_off: accende o spegne i pixel: PIXEL_ON o PIXEL_OFF (unsigned char)
 * @return void
 */
void GLCD_fill_rect (unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _on_off);

#if defined GLCD_KS0108B || (defined OLED_DRIVER_SH1106 && defined OLED_MODE_I2C)
/**
 * Disegna una cerchio di raggio _r
 * @param   _x_centr: coordinata x centro: 1~128 (unsigned char)   
 * @param   _y_centr: coordinata y centro: 1-64 (unsigned char)    
 * @param   _r: raggio 1-128 (unsigned char)          
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return  void
 * @note    Non implementata per OLED SPI e SSD1306 anche I2C
 */
void GLCD_draw_circle(unsigned char _x_centr, unsigned char _y_centr, unsigned char _r, unsigned char _on_off);

/**
 * Disegna un arco dato centro, raggio e angolo su display
 * @param   _x_centr: coordinata x centro: 1~128 (unsigned char) 
 * @param   _y_centr: coordinata y centro: 1-64 (unsigned char)    
 * @param   _r: raggio (unsigned char)
 * @param   _corner: angolo da disegnare
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return  void
 * @note    Non implementata per OLED SPI e SSD1306 anche I2C
 */
void GLCD_draw_arc(unsigned char _x_centr, unsigned char _y_centr, unsigned char _r, unsigned char _corner, unsigned char _on_off);
void GLCD_Plot8(unsigned char _x, unsigned char _y, unsigned char _dx, unsigned char _dy, unsigned char _corner, unsigned char _on_off);

/**
 * Disegna un cerchio pieno dato centro e raggio su display
 * @param   _x_centr: coordinata x centro: 1~128 (unsigned char) 
 * @param   _y_centr: coordinata y centro: 1-64 (unsigned char)    
 * @param   _r: raggio (unsigned char)
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return  void
 * @note    Non implementata per OLED SPI e SSD1306 anche I2C
 */
void GLCD_fill_circle(unsigned char _x_centr,unsigned char _y_center ,unsigned char _r, unsigned char _on_off);

/**
 * Disegna una rettangolo con angoli arrotondati su display
 * @param   _x_start: coordinata x di partenza: 1~128 (unsigned char)
 * @param   _y_start: coordinata y di partenza: 1~64 (unsigned char)
 * @param   _l: lunghezza: 1~128 (unsigned char)
 * @param   _h: altezza: 1~64 (unsigned char)
 * @param   _r: raggio degli angoli: 1~64 (unsigned char)
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return  void
 * @note    Non implementata per OLED SPI e SSD1306 anche I2C
 */
void GLCD_draw_rounded_rect(unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _r, unsigned char _on_off);

/**
 * Disegna una rettangolo pieno con angoli arrotondati su display
 * @param   _x_start: coordinata x di partenza: 1~128 (unsigned char)
 * @param   _y_start: coordinata y di partenza: 1~64 (unsigned char)
 * @param   _l: lunghezza: 1~128 (unsigned char)
 * @param   _h: altezza: 1~64 (unsigned char)
 * @param   _r: raggio degli angoli: 1~64 (unsigned char)
 * @param   _on_off: accende o spegne i pixel: ON o OFF (unsigned char)
 * @return  void
 * @note    Non implementata per OLED SPI e SSD1306 anche I2C
 */
void GLCD_fill_rounded_rect(unsigned char _x_start, unsigned char _y_start, unsigned char _l, unsigned char _h, unsigned char _r, unsigned char _on_off);
#endif

/**
 * Imposta le variabili a seconda del font selezionato
 * @param   *_font_pointer: puntatore all'array del font
 * @return  void
 */
void GLCD_set_font (const unsigned char *_font_pointer);

/**
 * Scrive un singlo carattere su GLCD
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 * @param   _unsigned character: carattere da scrivere (unsigned char)
 * @param   _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 * @param   _preserve_old: PRESERVE/OVERWRITE (unsigned char) Preserva o sovrascrive i dati sulla stessa pagina
 * @param   _on_off: scrittura positiva o negativa (unsigned char)
 * @return  valore di x effettivamente scritti (unsigned char)      
 * @pre     Impostare il font tramite la funzione TFT_set_font()
 * 
 */
unsigned char GLCD_write_char(unsigned char _x, unsigned char _y, unsigned char _character, unsigned char _savespace, unsigned char _preserve_old, unsigned char _on_off);

/**
 * Riempie lo spazio tra i caratteri con il colore sfondo e torna il valore di x effettivamente scritte.
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 * @param   _on_off: scrittura positiva o negativa (unsigned char)
 * @return  valore di x effettivamente scritti (unsigned char)
 * @note    Impostare il font tramite la funzione TFT_set_font()
 */
unsigned char GLCD_write_space_btw_chars (unsigned char _x, unsigned char _y, unsigned char _on_off);

/**
 * Cancella una colonna x
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 * @param   _on_off: scrittura positiva o negativa (unsigned char)
 * @return  void
 */
void GLCD_clear_1_col(unsigned char _x, unsigned char _y, unsigned char _on_off);

/**
 * Scrive una stringa di caratteri su GLCD
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 * @param   _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 * @param   _string: puntatore ad array di caratteri da scrivere, byte finale \0
 * @param   _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 * @param   _on_off: scrittura positiva o negativa (unsigned char)
 * @return  void
 */
void GLCD_write_string (unsigned char _x, unsigned char _y, unsigned char _max_x, const unsigned char _string[], unsigned char _savespace, unsigned char _on_off);

/**
 * Scrive un numero unsigned int su GLCD dopo averlo convertito in array stringa di caratteri ASCII
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 * @param   _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 * @param   _unsint: valore numerico unsigned int
 * @param   _number_digits: 1~5, numero di cifre da scrivere
 * @param   _zero_cleaning: ON/OFF, cancella zero iniziali se ON
 * @param   _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 * @param   _on_off: scrittura positiva o negativa (unsigned char)
 * @return  void
 */
void GLCD_write_unsint(unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned int _unsint, unsigned char _number_digits, unsigned char _zero_cleaning, unsigned char _savespace, unsigned char _on_off);

/**
 * Scrive un numero signed int su GLCD dopo averlo convertito in array stringa di caratteri ASCII
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 * @param   _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 * @param   _sint: valore numerico signed int
 * @param   _number_digits: 1~5, numero di cifre da scrivere
 * @param   _zero_cleaning: ON/OFF, cancella zero iniziali se ON
 * @param   _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 * @param   _on_off: scrittura positiva o negativa (unsigned char)
 * @return  void
 */
void GLCD_write_sint(unsigned char _x, unsigned char _y, unsigned char _max_x, signed int _sint, unsigned char _number_digits, unsigned char _zero_cleaning, unsigned char _savespace, unsigned char _on_off);

/**
 *  Scrive un numero signed int su GLCD dopo averlo convertito in array stringa di caratteri ASCII
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 * @param   _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 * @param   _pbcd: valore numerico codificato in packed BCD
 * @param   _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 * @param   _on_off: scrittura positiva o negativa (unsigned char)
 * @return  void
 */
void GLCD_write_PBCD(unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned char _pbcd, unsigned char _savespace, unsigned char _on_off);

/**
 * Scrive un byte HEX su display
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 * @param   _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 * @param   _hex: valore da scrivere (unsigned char)
 * @param   _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 * @param   _on_off: scrittura positiva o negativa (unsigned char)
 * @return  void     
 */
void GLCD_write_HEX(unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned char _hex, unsigned char _savespace, unsigned int _on_off);

/**
 * Scrive un byte come stringa di bit 1-0 su display
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 * @param   _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 * @param   _byte: valore da scrivere (unsigned char)
 * @param   _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 * @param   _on_off: scrittura positiva o negativa (unsigned char)
 * @return  void     
 */
void GLCD_write_bit_string (unsigned char _x, unsigned char _y, unsigned char _max_x, unsigned char _byte, unsigned char _savespace, unsigned int _on_off);

/**
 * Scrive un valore float su display
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~128, pixel pi a sinistra della scritta (unsigned char)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~64, pixel pi alto della scritta (unsigned char)
 * @param   _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~128 (unsigned char) 0 = PRESERVE OLD
 * @param   _float: valore da scrivere (float)
 * @param   _digits_int: numero di cifre interi da scrivere (1~5, unsigned char) 
 * @param   _digits_part: numero di cifre decimali da scrivere (1~10, unsigned char)
 * @param   _zero_cleaning: ON/OFF, cancella zero iniziali se ON
 * @param   _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (unsigned char)
 * @param   _on_off: scrittura positiva o negativa (unsigned char)
 * @return  void     
 */
void GLCD_write_float (unsigned char _x, unsigned char _y, unsigned char _max_x, float _float, unsigned char _digits_int, unsigned char _digits_part, unsigned char _zero_cleaning, unsigned char _savespace, unsigned int _on_off);